#!/bin/sh
#
# OracleASMDBClone_nonrac_tstamp.sh: shell script to automate cloning and bring up an Oracle database using backup image copies
#
# usage: OracleASMDBClone_nonrac_tstamp.sh <New database sid name> <Oracle Home path> <Disk group Name> <Oracle User> <Grid User> <log mount point> <timestamp yyyymmddhh24mi>
set -x
nsid=$1
oracleHome=$2
backupLocation=$3
osuser=$4
guser=$5
archmnt=$6
tstamp=$7
tdate="date +%m-%d-%Y:%H:%M:%S:"

if [ "$#" -lt 6 ]; then
 echo "This script needs 6 input parameter"
 echo "usage: OracleASMDBClone_nonrac_tstamp.sh <New database sid name> <Oracle Home path> <Disk group Name> <Oracle User> <Grid User> <log mount point> <timestamp yyyymmddhh24mi> "
 echo " For example: OracleASMDBClone_nonrac_tstamp.sh clonedb /home/oracle/app/oracle/product/11.2.0/dbhome_1 DATA1 oracle grid /act/mnt/logmnt 201504021435"
exit 1
fi

source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh

#logger for error messages
errMsgFile=$nsid"_errMsg"

function apmlogger {
     errmsg="$1"
     if [ ! -z "$errmsg" ]; then
        echo "$errmsg" > $errMsgFile
     fi
}

function apmappend {
     errmsg="$1"
     echo " $errmsg" >> $errMsgFile
}

function geterrmsg {
     ecode=$1
     if [ -z "$ecode" ]; then
        echo "missing error code when retrieving error message"
     fi
     msgrepo=/act/act_scripts/apmerrmsg.sh
     msg=`grep $ecode"=" $msgrepo | awk -F'=' '{print $NF}'`
     if [ ! -z "$msg" ]; then
        echo "$msg"
     else
        echo "failed to retrieve error message for $ecode"
     fi
}

function logerrormsg {
     ecode=$1
     appender=$2

     emsg=`geterrmsg $ecode`
     apmlogger "$emsg"
     if [ ! -z "$$appender" ]; then
        apmappend $appender
     fi
     echo `$tdate`$nsid": $emsg $appender"
}
#logger

if [ ! -z "$isrestore" ] && [ "$isrestore" = "true" ]; then
ORACLE_HOME=$oracleHome
export ORACLE_HOME
PATH=$ORACLE_HOME/bin:$PATH
export PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
ORACLE_SID=$nsid
export ORACLE_SID
TNS_ADMIN=$tnsadmindir
export TNS_ADMIN
ORACLE_UNQNAME=$nsid
export ORACLE_UNQNAME

if [ ! -z "$DB_TZ" ]; then
echo " ********** setting TZ:$DB_TZ  ***************"
TZ=$DB_TZ
export TZ
fi

su -m $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" << EOF
set heading off
set feedback off
startup mount;
alter database end backup;
exit;
EOF

echo "********** convert the tstamp to db timezone DB_TZ:$DB_TZ if different  ***************"
if [ ! -z "$DB_TZ" ]; then
tstamp=`su -m $osuser -c "$ORACLE_HOME/bin/sqlplus -S / as sysdba @/act/act_scripts/TZconvertApp.sql $tstamp $DB_TZ"`
retval=$?
TZ=$DB_TZ
export TZ
if [ $retval -gt 0 ]; then
echo "********** error converting the db timezone DB_TZ:$DB_TZ  ***************"
exit 110;
fi
fi

export NLS_DATE_FORMAT="yyyymmddhh24mi"
rmancmd="run
        {
        allocate channel d1 device type disk;
        allocate channel d2 device type disk;
        catalog start with '$archmnt/archivelog' noprompt;
        recover database until time \"to_date('$tstamp','yyyymmddhh24mi')\";
        release channel d1;
        release channel d2;
        }"

dbrecover="/act/act_scripts/asmclone/dbrecover_"$nsid".txt"
if [ ! -f "$dbrecover" ]; then
   touch $dbrecover
   change_own $osuser $dbrecover
fi
ORACLE_HOME=$oracleHome
export ORACLE_HOME
PATH=$ORAHOME/bin:$PATH
export PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
ORACLE_SID=$nsid
export ORACLE_SID
TNS_ADMIN=$tnsadmindir
export TNS_ADMIN
retv1=`su -m $osuser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << EOF
                 $rmancmd
                 exit
EOF`

if [ $? -ne 0 ]; then
   logerrormsg EM_DB_RECOVERY
   exit 1
fi

su -m $osuser -c "$ORACLE_HOME/bin/sqlplus -s / as sysdba" << EOF
set heading off
set feedback off
alter database open resetlogs;
exit;
EOF
if [ $? -ne 0 ]; then
   logerrormsg EM_DB_START $nsid
   exit 1
fi

exit 0
#inband restore
fi

source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh

oratab=/etc/oratab
gridhome=`get_grid_home $oratab`
if [ $? -ne 0 ]; then
   logerrormsg EM_GET_GRID_HOME
   exit 1
fi

if [ ! -d "$gridhome" ]; then
   logerrormsg EM_GRID_HOME $gridhome
   exit 1
fi

asmsid=`grep -v "^[#]" $oratab |grep -i +asm | cut -d":" -f1`
export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$gridhome
export ORACLE_SID=$asmsid
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

psid=`get_asm_sid $backupLocation $guser`
remountflag=$isremount

echo "******************* remount flag: $remountflag ********************"
echo "******************* psid: $psid ********************"
if [ "$remountflag" = "true" ]; then
#if [ $? -eq 0 ] && [ ! -z "$ischild" ] && [ "$ischild" = "false" ]; then
#remount of existing appaware mounted diskgroup
/act/act_scripts/asmclone/OracleASMDBRemount_nonrac.sh $nsid $oracleHome $backupLocation $osuser 
   if [ $? -ne 0 ]; then
      logerrormsg EM_REMOUNT $nsid
      exit 1
   fi
else
#regular appAware mount

BASEDIR=$(dirname $0)
echo $BASEDIR
cd $BASEDIR
# correctly set the ORACLE_BASE AND ORACLE_HOME  for your environment
export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
#ORACLE_BASE=`echo $ORACLE_HOME | awk -F"/product" '{print $(NF-1)}'`
#export ORACLE_BASE=$ORACLE_BASE
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

change_own $osuser $BASEDIR

if [ ! -d $ORACLE_HOME ]; then
   logerrormsg EM_ORACLE_HOME
   exit 1
fi

#if [ ! -d $ORACLE_BASE ]; then
#   echo "could not find ORACLE BASE"
#   exit 1
#fi

backupLocation=`echo $backupLocation | tr '[a-z]' '[A-Z]'`
if [ ! -d /act/touch/$backupLocation ]; then
   backupLocation=`echo $backupLocation | tr '[A-Z]' '[a-z]'`
   if [ ! -d /act/touch/$backupLocation ]; then
      logerrormsg EM_FIND_BACKUP_LOCATION $backupLocation
      exit 1
   fi
fi
dataDir=$backupLocation

# get the backup parameter file location
mname=$backupLocation
echo $mname
pfileloc=/act/touch/$mname

# get the production sid name from mount
getpname=`ls -ltr $pfileloc | grep '__backup.ora'`
echo $getpname
pname=`echo $getpname | awk '{print $NF}'`
echo $pname
osid=`echo $pname | awk -F"__" '{print $(NF-1)}'`
echo "***************"
echo $osid

if [ -z "$osid" ]; then
   logerrormsg EM_FIND_ORIG_SID
   exit 1
fi

dbfsPath=/act/touch/$backupLocation/$osid"_datafiles_readonly_catalog.conf"
echo "read only tablespace file path : $dbfsPath"

origfile=$pfileloc/$osid"__backup.ora"
cp $origfile $pfileloc/$osid"__cpfile.ora"
chmod 775 $pfileloc/$osid"__cpfile.ora"
orafile=$pfileloc/$osid"__cpfile.ora"
orarollbacktemp=$pfileloc/$osid"__cpfilerollback.ora"
echo $orafile

openDatabase="openDatabase_"$nsid".sql"
openPDatabase="openPDatabase_"$nsid".sql"
openDatabase1="openDatabase1_"$nsid".sql"
openTmpDatabase="openTmpDatabase_"$nsid".sql"
openTmpDatabase1="openTmpDatabase1_"$nsid".sql"
recoverDatabase="recoverDatabase_"$nsid".sql"
verifyDatabase="verifyDatabase"_$nsid".sql"
runnidsql="runnidrotbs"_$nsid".sql"

CLONE_CRT_SQL="Clone_createCtrlFileASM_"$nsid".sql"
addUnnamed="addUnnamed_"$nsid".sql"
tabfilelist="tabfilelist_"$nsid
undofilelist="undofilelist_"$nsid".sql"
offlinefilelist="offlinefilelist_"$nsid
pluggablelist="pluggablelist_"$nsid

pglist="pglist_"$nsid

sqlplus=$ORACLE_HOME/bin/sqlplus

var="+ASM"
for ASMSID in $var; do
ASMS=`grep -v "^[#]" /etc/oratab | grep -i "$ASMSID" | head -n1 | cut -f1 -d":"`
 echo "asmsid is: $ASMS"
ASMHOME=`grep -v "^[#]" /etc/oratab | grep -i "$ASMS:"  | head -n1 | cut -f2 -d":"`
 echo "asmhome is: $ASMHOME"
done

export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$ASMHOME
export LD_LIBRARY_PATH=$ASMHOME/lib:/usr/lib:/lib
export PATH=$ASMHOME/bin:$PATH
export ORACLE_SID="$ASMS"
asmcmd=$ASMHOME/bin/asmcmd

#set appware generation
if [ "$ischild" = "true" ]; then
   pgen=`get_apm_latest_gen $backupLocation $guser`
   gen=$(( pgen + 1 ))
else
   gen=0
fi

apmtag="apmgen_"$gen"_f"
#

su -m $guser -c $asmcmd << EOF
cd $mname
mkdir $nsid  
cd $nsid
mkdir datafile
mkdir archivelog
mkdir controlfile
mkdir onlinelog
mkdir $apmtag
EOF

scname=`su -m $guser srvctl config scan | head -n1 | cut -f2 -d":" | cut -f1 -d","`
pnum=`su -m $guser srvctl config scan_listener | grep -i "TCP:" | head -n1 | cut -f3 -d ":"`

pnum=`echo $pnum | cut -f1 -d ","`

scname=$(tr -d ' ' <<< "$scname")

#scname=`srvctl config scan | head -n1 | cut -f2 -d":" | cut -f1 -d","`
#pnum=`srvctl config scan_listener | grep -i "TCP:" | head -n1 | cut -f3 -d ":"`

if [ -z $nsid ]; then
   logerrormsg EM_NEW_SID
   exit 1
fi

if [ -z $orafile ]; then
   logerrormsg EM_INIT_FILE
   exit 1
fi

#oobase=$ORACLE_BASE
#echo "Oracle base is $ORACLE_BASE"

thcnt=`grep -i ".thread" $orafile | wc -l`
undocnt=`grep -i ".undo_tablespace" $orafile | wc -l`

# replace SID with new SID
sed -i "s/'$osid'/'$nsid'/g" $orafile

export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

#make required directory structures
if [ -z "$diagnostic_dest" ]; then
  su -m $osuser -c "mkdir -p $ORACLE_HOME/../../admin/$nsid/adump"
  su -m $osuser -c "mkdir -p $ORACLE_HOME/../../diag"
else
  su -m $osuser -c "mkdir -p $diagnostic_dest/$nsid"
  su -m $osuser -c "mkdir -p $diagnostic_dest/$nsid/admin/adump"
fi

if [[ x$scname == "x" || x$pnum == "x" ]]; then
sed -i '/remote_listener/d' $orafile
else
remotelistener="*.remote_listener='$scname:$pnum'"
sed -i '/remote_listener/d' $orafile
echo -e "$remotelistener" >> $orafile
fi

#ctlfile=`su -m $guser $asmcmd ls +$dataDir/$SID/controlfile`
dataDir1="+"$dataDir
controlfile="*.control_files='$dataDir1'"
dbcreatefile="*.db_create_file_dest='$dataDir1'"
dbrecoveryfile="*.db_recovery_file_dest='$dataDir1'"
#logarchivedest="*.log_archive_dest_1='LOCATION=$dataDir/$nsid/archivelog/'"

nsidlen=`echo -n $nsid | wc -m`
if [ $nsidlen -gt 8 ]; then
newdb_name=`echo $nsid | cut -c1-8`
db_unique_name=$nsid
else
newdb_name=$nsid
fi

dbname="*.db_name='$newdb_name'"
dbuniquename="*.db_unique_name='$nsid'"

jqprocess="*.job_queue_processes=0"
clustdb="*.cluster_database=false"
tmpundo="*.temp_undo_enabled=false"

hgenabled=`cat /sys/kernel/mm/transparent_hugepage/enabled | cut -d "[" -f2 | cut -d "]" -f1`
if [ "$hgenabled" = "always" ]; then
 if [ -z "$sgapct" ]; then
 sgapct=80
 fi
fi

if [ -z "$totalmemory" ]; then
    #sgatarget="*.sga_target=800m"
    #pgatarget="*.pga_aggregate_target=400m"
    echo " memory size is not defined "
    echo "source memory setup will be used"
else
    if [ ! -z "$totalmemory" ] && [ -z "$sgapct" ]; then
        echo " SGA is not defined "
        echo "memory_target setup will be used"
        sed -i '/sga_max_size=/d' $orafile
        sed -i '/sga_target=/d' $orafile
        sed -i '/pga_aggregate_target=/d' $orafile
        sed -i '/memory_target=/d' $orafile
        sed -i '/memory_max_target=/d' $orafile
        sed -i '/shared_pool_size=/d' $orafile
        sed -i '/streams_pool_size=/d' $orafile
        sed -i '/db_cache_size/d' $orafile
        sed -i '/db_keep_cache_size/d' $orafile
        sed -i '/shared_pool_reserved_size=/d' $orafile
        sed -i '/large_pool_size/d' $orafile
        totalmemoryM=$totalmemory"m"
        mtarget="*.memory_target=$totalmemoryM"
        echo -e "$mtarget" >> $orafile
        echo "1pga: $pgatarget" 
        echo "1sga: $sgatarget" 
        echo "1totalmemory: $totalmemory"
        echo "1sgapct: $sgapct"
    else
        echo " total memory and SGA is defined "
        echo "sga and pga setup will be used"
        sed -i '/sga_max_size=/d' $orafile
        sed -i '/sga_target=/d' $orafile
        sed -i '/pga_aggregate_target=/d' $orafile
        sed -i '/memory_target=/d' $orafile
        sed -i '/memory_max_target=/d' $orafile
        sed -i '/shared_pool_size=/d' $orafile
        sed -i '/streams_pool_size=/d' $orafile
        sed -i '/db_cache_size/d' $orafile
        sed -i '/db_keep_cache_size/d' $orafile
        sed -i '/shared_pool_reserved_size=/d' $orafile
        sed -i '/large_pool_size/d' $orafile
        sgc=$(($totalmemory*$sgapct/100))
        sgam=$sgc"m"
        sgatarget="*.sga_target=$sgam"
        pgc=$(($totalmemory - $sgc ))
        pgam=$pgc"m"
        pgatarget="*.pga_aggregate_target=$pgam"
        echo -e "$sgatarget" >> $orafile
        echo -e "$pgatarget" >> $orafile
        lps=$((($sgc*10)/100))
        if [ "$lps" -gt "200" ]; then
           lapsm=$lps"m"
        else
           lapsm=200m
        fi

        largepool="*.large_pool_size=$lapsm"
        echo -e "$largepool" >> $orafile

        echo "pga: $pgatarget" 
        echo "sga: $sgatarget" 
        echo "totalmemory: $totalmemory"
        echo "sgapct: $sgapct"
        if [ ! -z "$shared_pool_size" ]; then
            sps=$shared_pool_size"m"
            sharedpoolsize="*.shared_pool_size=$sps"
            echo -e "$sharedpoolsize" >> $orafile
        fi

        if [ ! -z "$db_cache_size" ]; then
           dcs=$db_cache_size"m"
           dbcachesize="*.db_cache_size=$dcs"
           echo -e "$dbcachesize" >> $orafile
        fi
    fi
fi

sed -i '/db_recovery_file_dest_size=/d' $orafile
if [ ! -z "$db_recovery_file_dest_size" ]; then
drfds=$db_recovery_file_dest_size"m"
dbrecoveryfilesize="*.db_recovery_file_dest_size=$drfds"
else
dbrecoveryfilesize="*.db_recovery_file_dest_size=50g"
fi

sed -i '/processes=/d' $orafile
if [ ! -z "$processes" ]; then
process="*.processes=$processes"
else
process="*.processes=500"
fi

if [ ! -z "$open_cursors" ]; then
sed -i '/open_cursors=/d' $orafile
opencursors="*.open_cursors=$open_cursors"
echo -e "$opencursors" >> $orafile
fi

if [ ! -z "$db_block_size" ]; then
sed -i '/db_block_size=/d' $orafile
dbblocksize="*.db_block_size=$db_block_size"
echo -e "$dbblocksize" >> $orafile
fi

sed -i '/oracle_base=/d' $orafile
sed -i '/inmemory_/d' $orafile
sed -i '/use_large_pages=/d' $orafile
sed -i '/control_files=/d' $orafile
sed -i '/audit_file_dest=/d' $orafile
sed -i '/db_create_file_dest=/d' $orafile
sed -i '/db_recovery_file_dest=/d' $orafile
sed -i '/diagnostic_dest=/d' $orafile
sed -i '/log_archive_dest_1=/d' $orafile
sed -i '/log_archive_dest/d' $orafile
sed -i '/cluster_database=/d' $orafile
#sed -i '/remote_listener/d' $orafile
sed -i '/local_listener/d' $orafile
sed -i '/user_dump_dest/d' $orafile
sed -i '/core_dump_dest/d' $orafile
sed -i '/db_name/d' $orafile
sed -i '/dispatchers=/d' $orafile
sed -i '/SPFILE=/d' $orafile 

sed -i '/thread=/d' $orafile
sed -i '/THREAD=/d' $orafile
sed -i '/instance_number=/d' $orafile
sed -i '/undo_tablespace=/d' $orafile
sed -i '/__/d' $orafile
sed -i '/service_names=/d' $orafile
sed -i '/sga_max_size=/d' $orafile
sed -i '/db_create_online_log_dest/d' $orafile
sed -i '/streams_pool_size/d' $orafile

sed -i '/dg_broker_config_file/d' $orafile
sed -i '/dg_broker_start/d' $orafile
sed -i '/fal_client/d' $orafile
sed -i '/fal_server/d' $orafile
sed -i '/log_archive_config/d' $orafile
sed -i '/log_archive_trace/d' $orafile
sed -i '/log_archive_min_succeed_dest/d' $orafile
sed -i '/log_file_name_convert/d' $orafile
sed -i '/standby_file_management/d' $orafile
sed -i '/db_file_name_convert/d' $orafile
sed -i '/archive_lag_target/d' $orafile
sed -i '/log_archive_trace/d' $orafile

sed -i '/cluster_interconnects/d' $orafile
sed -i '/_diag_adr_trace_dest/d' $orafile

sed -i '/cluster_database_instances/d' $orafile
sed -i '/instance_name/d' $orafile
sed -i '/db_unique_name/d' $orafile
sed -i '/standby_archive_dest/d' $orafile
sed -i '/background_dump_dest/d' $orafile

sed -i '/_fix_control/d' $orafile
sed -i '/event/d' $orafile

sed -i '/enable_goldengate_replication/d' $orafile
sed -i '/_adg_parselock_timeout/d' $orafile
sed -i '/parallel_min_servers/d' $orafile
sed -i '/parallel_max_servers/d' $orafile
sed -i '/parallel_servers_target/d' $orafile

sed -i '/use_large_pages/d' $orafile

chmod 755 $orafile

echo -e "$controlfile" >> $orafile
echo -e "$dbname" >> $orafile
echo -e "$dbuniquename" >> $orafile
#echo -e "$auditfile" >> $orafile
echo -e "$dbcreatefile" >> $orafile
echo -e "$dbrecoveryfile" >> $orafile
echo -e "$dbrecoveryfilesize" >> $orafile
if [ ! -z "$diagnostic_dest" ]; then
 dbdiagnostic="*.diagnostic_dest='$diagnostic_dest'"
 echo -e "$dbdiagnostic" >> $orafile
 adest="*.audit_file_dest='$diagnostic_dest/$nsid/admin/adump'"
 echo -e "$adest" >> $orafile
fi
#echo -e "$logarchivedest" >> $orafile
echo -e "$process" >> $orafile
echo -e "$clustdb" >> $orafile
#echo -e "$remotelistener" >> $orafile
echo -e "$jqprocess" >> $orafile
tmpundocnt=0
tmpundocnt=`grep ".temp_undo_enabled" $orafile | wc -l`
if [ "$tmpundocnt" -gt "0" ]; then
sed -i '/temp_undo_enabled/d' $orafile
echo -e "$tmpundo" >> $orafile
fi

sed -i '/disk_asynch_io/d' $orafile
diskasyncio="*.disk_asynch_io='TRUE'"
echo -e "$diskasyncio" >> $orafile

chmod 755 $orafile

#ncount=`grep "ASM_" asm_node_config.conf | cut -f2 -d":" | wc -l`

#var=`grep "ASM_" $BASEDIR/asm_node_config.conf | cut -f2 -d":"`

# generate a controlfile creation script
./generateASMCtrlFileScript.sh $nsid $dataDir $osid $guser

retval=$?
if [ $retval -gt 0 ]; then
   logerrormsg EM_GENERATE_CONTROLFILE
   exit 1;
fi

#create sql file to open the database and add tempfile
if [ -e $openDatabase ]; then
   #echo "removing $openDatabase before generating a new one"
   rm -f $openDatabase
fi

if [ -e $openDatabase1 ]; then
  # echo "removing $openDatabase1 "
   rm -f $openDatabase1
fi

if [ -e $recoverDatabase ]; then
   #echo "removing $recoverDatabase before generating a new one"
   rm -f $recoverDatabase
fi

if [ -e $addUnnamed ]; then
   #echo "removing $openDatabase before generating a new one"
   rm -f $addUnnamed
fi

if [ -e $tabfilelist ]; then
   #echo "removing $tabfilelist before generating a new one"
   rm -f $tabfilelist
fi

if [ -e $undofilelist ]; then
  # echo "removing $undofilelist "
   rm -f $undofilelist
fi
if [ -e $orarollbacktemp ]; then
  # echo "removing $orarollbacktemp "
   rm -f $orarollbacktemp
fi

# now export new SID, do this outside of shell, before invoke this shell
export ORACLE_SID=$nsid
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

# copy ora file
#initf1=$ORACLE_HOME/dbs/"init"$nsid".ora"
initf=$ORACLE_HOME/dbs/"init"$nsid".ora"

# keep a copy of ora file under ORACLE_BASE
cp $orafile $ORACLE_HOME/../../"init"$nsid".ora"
#check if spfile<sid>.ora exist, and move it if true
spf_fs=$ORACLE_HOME/dbs/"spfile"$nsid".ora"
if [ -f $spf_fs ]; then
   mvts=`date +%Y%m%d`
   su -m $osuser -c "mv $spf_fs $ORACLE_HOME/dbs/spfile$nsid.ora-$mvts"
fi

# start the database for cloning
spf=$dataDir1/$nsid/"spfile"$nsid".ora"
su -m $osuser -c "$sqlplus / as sysdba @createPfile.sql $orafile $spf "

#su -m $osuser -c "echo 'SPFILE=$spf' > $initf1"
su -m $osuser -c "echo 'SPFILE=$spf' > $initf"

change_own $osuser $CLONE_CRT_SQL

if [ ! -z "$DB_TZ" ]; then
echo " ********** setting TZ:$DB_TZ  ***************"
TZ=$DB_TZ
export TZ
fi

ORACLE_UNQNAME=$nsid
export ORACLE_UNQNAME
su -m $osuser -c "sqlplus / as sysdba @mountDatabase.sql $CLONE_CRT_SQL"

retval=$?
if [ $retval -gt 0 ]; then
   logerrormsg EM_MOUNT_DATABASE  
   exit 1;
fi

export ORACLE_SID=$nsid
opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
select count(*) from v\$database where open_mode = 'MOUNTED';
exit;
EOF1`
retval=$?

if [ $retval -gt 0 ]; then
 logerrormsg EM_DB_MOUNT
 exit 1;
elif [ $opencnt -eq 0 ]; then
 logerrormsg EM_DB_MOUNT
 exit 1;
fi

echo "********** convert the tstamp to db timezone DB_TZ:$DB_TZ if different  ***************"
if [ ! -z "$DB_TZ" ]; then
tstamp=`su -m $osuser -c "sqlplus -S / as sysdba @/act/act_scripts/TZconvertApp.sql $tstamp $DB_TZ"`
retval=$?
if [ $retval -gt 0 ]; then
echo "********** error converting the db timezone DB_TZ:$DB_TZ  ***************"
exit 110;
fi
fi

if [ -e $CLONE_CRT_SQL ]; then
   echo $CLONE_CRT_SQL
   rm -f $CLONE_CRT_SQL
fi

export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
export ORACLE_SID=$nsid

dbrecover="dbrecover_"$nsid".txt"
export ORACLE_SID=$nsid
catpath=$dataDir1"/"$osid
nsid1=$nsid"1"
#diffcnt=$(($ncount - $undocnt))

vnum=0
vnum=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feedback off
select count(*) from v\$instance where version like '12%';
exit;
EOF1`
echo "********************* vnum=$vnum ***********"
pdbcnt=0
if [ "$vnum" -gt "0" ]; then
  pdbcnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
  set head off;
  set feedback off
  select count(*) from v\$pdbs where name like 'PDB%SEED';
  exit;
EOF1`
fi

export vnum
export pdbcnt
echo "******* recovering database *********** "
echo " recovering database :pdbcnt=$pdbcnt ischild=$ischild " > $dbrecover
change_own $osuser $dbrecover

#if [ $pdbcnt -gt 0 ]; then
su -m $osuser -c "$sqlplus / as sysdba " << EOF
alter database archivelog;
exit;
EOF
#fi

ORACLE_UNQNAME=$nsid
export ORACLE_UNQNAME
./recoverRman_tstamp.sh $catpath $nsid $archmnt $osuser $tstamp $dbfsPath

if [ $pdbcnt -eq 0 ]; then
su -m $osuser -c "$sqlplus / as sysdba @newDatafile.sql"
nstmtsql=nstmtsql.sql
echo "set head off;" > $nstmtsql
echo "set feedback off" >> $nstmtsql
echo "select to_char(to_date('$tstamp'+1,'yyyymmddhh24mi'),'yyyymmddhh24mi') from dual;" >> $nstmtsql
echo "exit;" >> $nstmtsql
change_own $osuser $nstmtsql
ntstamp=`su -m $osuser -c "sqlplus -S / as sysdba @$nstmtsql"`
rm -f $nstmtsql
./recoverRman_tstamp.sh $catpath $nsid $archmnt $osuser $ntstamp $dbfsPath
fi

if [ "$pdbcnt" -gt "0" ]; then
su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $offlinefilelist
set head off;
set feed off;
set echo off;
select file# || ':' || ltrim(substr(name,instr(name,'/',-1)),'/')  from v\$datafile where status='RECOVER' and enabled = 'READ WRITE';
exit;
EOF
chmod 755 $offlinefilelist

while read line
do
    set -- $line
    filenum=$1
        echo "filenumber is: $filenum"
if [ "x$filenum" != "x" ]; then
fnm=`echo $filenum | awk -F':' '{print $1}'`
fname=`echo $filenum | awk -F':' '{print $NF}'`
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << EOF
run {
SET NEWNAME FOR DATAFILE $fnm TO '$catpath/datafile/$fname';
alter database create datafile $fnm as new;
}
exit;
EOF
fi
done < $offlinefilelist
fi

if [ $pdbcnt -gt 0 ]; then
su -m $osuser -c "$sqlplus / as sysdba @newDatafile.sql"
nstmtsql=nstmtsql.sql
echo "set head off;" > $nstmtsql
echo "set feedback off" >> $nstmtsql
echo "select to_char(to_date('$tstamp'+1,'yyyymmddhh24mi'),'yyyymmddhh24mi') from dual;" >> $nstmtsql
echo "exit;" >> $nstmtsql
change_own $osuser $nstmtsql
ntstamp=`su -m $osuser -c "sqlplus -S / as sysdba @$nstmtsql"`
rm -f $nstmtsql
./recoverRman_tstamp.sh $catpath $nsid $archmnt $osuser $ntstamp $dbfsPath
fi

echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $openDatabase
echo -e "alter database open resetlogs;" >> $openDatabase
echo -e "CREATE temporary TABLESPACE TEMP_CL TEMPFILE '$dataDir1/$nsid/temp_cl01.dbf' size 5g autoextend on maxsize unlimited;" >> $openDatabase
echo -e "exit" >> $openDatabase

echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $openPDatabase
echo -e "alter database open;" >> $openPDatabase
echo -e "CREATE temporary TABLESPACE TEMP_CL TEMPFILE '$dataDir1/$nsid/temp_cl01.dbf' size 5g autoextend on maxsize unlimited;" >> $openPDatabase
echo -e "shutdown immediate;" >> $openPDatabase
echo -e "startup mount;" >> $openPDatabase
echo -e "recover database until cancel;" >> $openPDatabase
echo -e "alter database open resetlogs;" >> $openPDatabase
echo -e "exit" >> $openPDatabase

change_own $osuser $openDatabase
change_own $osuser $openPDatabase

openDBlog="openDBlog_"$nsid".txt"

if [ -e $openDBlog ]; then
#   echo "removing $openDBlog before generating a new one"
   rm -f $openDBlog
fi

echo -e "Opening Database is started..." > $openDBlog

change_own $osuser $openDBlog

if [ "$pdbcnt" -gt "0" ]; then
su -m $osuser -c "$sqlplus / as sysdba @recoverPDBSeed.sql" >> $openDBlog
fi

nsid1=$nsid"1"
export ORACLE_SID=$nsid
su -m $osuser -c "sqlplus / as sysdba @$openDatabase " >> $openDBlog
retval1=$?

orabug=`grep ORA-01092 $openDBlog`
if [ -z "$orabug" ]; then
orabug=`grep ORA-03113 $openDBlog`
fi

if [ ! -z "$orabug" ]; then
sleep 60
export ORACLE_SID=$nsid
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << EOF
run {
startup mount;
recover database;
}
exit;
EOF
fi
if [ "$pdbcnt" -gt "0" ]; then
su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $offlinefilelist
set head off;
set feed off;
set echo off;
select file# from v\$datafile where status='OFFLINE';
exit;
EOF
chmod 755 $offlinefilelist

while read line
do
    set -- $line
    filenum=$1
        echo "filenumber is: $filenum"
if [ "x$filenum" != "x" ]; then
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << EOF
run {
recover datafile $filenum;
alter database datafile $filenum online;
}
exit;
EOF
fi
done < $offlinefilelist
fi
if [ ! -z "$orabug" ]; then
su -m $osuser -c "sqlplus / as sysdba @$openPDatabase " >> $openDBlog
retval1=$?
fi

if [ "$retval1" -gt "0" ]
then
 while [ "$retval1" -gt "0" ]; do

undfcnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
select count(*) from v\$datafile where name like '%/UNNAMED%';
exit;
EOF1`

retval1=0

  if [ "$undfcnt" -gt "0" ]; then

      undfnum=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF2'
        set head off;
        select file# from v\$datafile where name like '%/UNNAMED%';
        exit;
EOF2`

        echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $addUnnamed
        echo -e "alter database create datafile $undfnum as new;" >> $addUnnamed
        echo -e "exit" >> $addUnnamed
        change_own $osuser $addUnnamed

        su -m $osuser -c "sqlplus / as sysdba @$addUnnamed "
       ./recoverRman_tstamp.sh $catpath $nsid $archmnt $osuser $tstamp $dbfsPath
        su -m $osuser -c "sqlplus / as sysdba @$openDatabase " >> $openDBlog
        retval1=$?;
        orabug=`grep ORA-01092 $openDBlog`
        if [ -z "$orabug" ]; then
                orabug=`grep ORA-03113 $openDBlog`
        fi
        if [ ! -z "$orabug" ]; then
                echo "******************" >> $openDBlog
                echo "ACT_PDB_TF" >> $openDBlog
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append  " << EOF
run {
startup mount;
recover database;
}
exit;
EOF
                su -m $osuser -c "sqlplus / as sysdba @$openDatabase " >> $openDBlog
                retval1=$?
                oraissue=`awk '/ACT_PDB_TF/{y=1;next}y' $openDBlog | grep ORA-01196`
                if [ ! -z "$oraissue" ]; then
                   logerrormsg EM_DB_OPEN "$oraissue"
                   exit 1
                   #exit $oraissue
                fi
         fi
   else
     retval1=0
   fi
 done
fi

ggusername=`su -m $osuser -c "$sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none;
select distinct owner from dba_segments where segment_name like 'GGS_%' and owner<>'SYS' and owner<>'SYSTEM' and rownum<2;
exit;
EOF1`

if [ ! -z "$ggusername" ]; then
echo "Golden Gate User:$ggusername"
su -m $osuser -c "sqlplus -S / as sysdba " <<EOF1
WHENEVER SQLERROR EXIT SQL.SQLCODE
alter user "$ggusername" account lock;
startup force restrict;
drop user "$ggusername" cascade;
exit;
EOF1
retval=$?
echo "Drop User: $retval"

if [ $retval -gt 0 ]; then
su -m $osuser -c "sqlplus -S / as sysdba " <<EOF1
WHENEVER SQLERROR EXIT SQL.SQLCODE
shutdown abort;
startup force restrict;
drop user "$ggusername" cascade;
exit;
EOF1
fi
fi


opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?
if [ $retval -gt 0 ]; then
su -m $osuser -c "$sqlplus / as sysdba " << EOF
startup force restrict;
exit;
EOF
fi

# drop old undo tablespace
ponum=0
ponum=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feedback off
select count(*) from dba_rollback_segs where status = 'PARTLY AVAILABLE' or status = 'NEEDS RECOVERY';
exit;
EOF1`

su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $undofilelist
set head off;
set feed off;
set echo off;
set newpage none;
select distinct tablespace_name from dba_undo_extents;
exit;
EOF

if [ "$ponum" -gt "0" ]; then
undorollofflist=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feed off;
set echo off;
set linesize 4000;
set newpage none;
select '*._offline_rollback_segments='||ltrim(rtrim(listagg(to_char(SEGMENT_NAME),',') WITHIN GROUP (ORDER BY SEGMENT_NAME ))) from dba_rollback_segs where status = 'PARTLY AVAILABLE' or status = 'NEEDS RECOVERY';
exit;
EOF1`
fi

echo -e "create undo tablespace UNDOTBS_DEF datafile '$dataDir1' size 500m autoextend on;" > $openTmpDatabase
echo -e "alter system set undo_tablespace=UNDOTBS_DEF scope=both;" >> $openTmpDatabase
echo -e "exit" >> $openTmpDatabase

spf=$dataDir1/$nsid/"spfile"$nsid".ora"
su -m $osuser -c "$sqlplus / as sysdba @createTmpPfile.sql $spf $orarollbacktemp "
chmod 755 $orarollbacktemp

if [ "$ponum" -gt "0" ]; then
echo -e $undorollofflist >> $orarollbacktemp

fi
ttbs="*.undo_tablespace=UNDOTBS_DEF"
sed -i '/undo_tablespace/d' $orarollbacktemp
echo -e $ttbs >> $orarollbacktemp
chmod 755 $orarollbacktemp

chmod 755 $openTmpDatabase
export ORACLE_SID=$nsid
su -m $osuser -c "sqlplus / as sysdba @$openTmpDatabase " >> $openDBlog


echo -e "shutdown immediate;" > $openTmpDatabase1
echo -e "startup force restrict pfile='$orarollbacktemp'" >> $openTmpDatabase1
echo -e "alter system set undo_tablespace=UNDOTBS_DEF;" >> $openTmpDatabase1

while read line
do
    set -- $line
    undoname=$1
        echo "outside if $undoname"
        if [ "x$undoname" != "x" ]; then
        echo "inside if $undoname"
        echo "drop tablespace $undoname including contents and datafiles;" >> $openTmpDatabase1
        fi
done < $undofilelist

echo -e "shutdown immediate;" >> $openTmpDatabase1
echo -e "startup force;" >> $openTmpDatabase1
echo -e "exit" >> $openTmpDatabase1

chmod 755 $openTmpDatabase1

echo "******************" >> $openDBlog
echo "ACT_PDB_TF" >> $openDBlog

export ORACLE_SID=$nsid
su -m $osuser -c "sqlplus / as sysdba @$openTmpDatabase1 " >> $openDBlog
retval=$?

oraissue=`awk '/ACT_PDB_TF/{y=1;next}y' $openDBlog | grep ORA-03113`
if [ -z "$oraissue" ]; then
oraissue=`awk '/ACT_PDB_TF/{y=1;next}y' $openDBlog | grep ORA-01548`
fi
if [ ! -z "$oraissue" ]; then
sleep 60
echo -e "startup force restrict;" > $openTmpDatabase
echo -e "exit" >> $openTmpDatabase

chmod 755 $openTmpDatabase

export ORACLE_SID=$nsid
su -m $osuser -c "sqlplus / as sysdba @$openTmpDatabase " >> $openDBlog
ponum=0
ponum=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feedback off
select count(*) from dba_rollback_segs where status = 'PARTLY AVAILABLE' or status = 'NEEDS RECOVERY';
exit;
EOF1`

if [ "$ponum" -gt "0" ]; then
undorollofflist=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feed off;
set echo off;
set linesize 4000;
set newpage none;
select '*._offline_rollback_segments='||ltrim(rtrim(listagg(to_char(SEGMENT_NAME),',') WITHIN GROUP (ORDER BY SEGMENT_NAME ))) from dba_rollback_segs where status = 'PARTLY AVAILABLE' or status = 'NEEDS RECOVERY';
exit;
EOF1`
fi

sed -i '/offline_rollback_segments/d' $orarollbacktemp
if [ "$ponum" -gt "0" ]; then
echo -e $undorollofflist >> $orarollbacktemp
fi

ttbs="*.undo_tablespace=UNDOTBS_DEF"
sed -i '/undo_tablespace/d' $orarollbacktemp
echo -e $ttbs >> $orarollbacktemp
chmod 755 $orarollbacktemp

echo -e "shutdown immediate" > $openTmpDatabase1
echo -e "startup force restrict pfile='$orarollbacktemp'" >> $openTmpDatabase1
echo -e "alter system set undo_tablespace=UNDOTBS_DEF;" >> $openTmpDatabase1

while read line
do
    set -- $line
    undoname=$1
        echo "outside if $undoname"
        if [ "x$undoname" != "x" ]; then
        echo "inside if $undoname"
        echo "drop tablespace $undoname including contents and datafiles;" >> $openTmpDatabase1
        fi
done < $undofilelist

echo -e "shutdown immediate;" >> $openTmpDatabase1
echo -e "startup force;" >> $openTmpDatabase1
echo -e "exit" >> $openTmpDatabase1

chmod 755 $openTmpDatabase1
sleep 60
export ORACLE_SID=$nsid
su -m $osuser -c "sqlplus / as sysdba @$openTmpDatabase1 " >> $openDBlog
orasmon=`grep ORA-01154 $openDBlog`
if [ ! -z "$orasmon" ]; then
 sed -i '0,/shutdown immediate/s//shutdown abort/' $openTmpDatabase1
 su -m $osuser -c "sqlplus / as sysdba @$openTmpDatabase1 " >> $openDBlog
fi
fi

if [ "$pdbcnt" -gt "0" ]; then
su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $pluggablelist
set head off;
set feed off;
set echo off;
select name from v\$pdbs where con_id in (select con_id from v\$recover_file where error = 'UNKNOWN ERROR');
exit;
EOF
chmod 755 $pluggablelist

while read line
do
    set -- $line
    filenum=$1
        echo "filenumber is: $filenum"
if [ "x$filenum" != "x" ]; then
su -m $osuser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << EOF
run {
recover pluggable database $filenum;
}
exit;
EOF
fi
done < $pluggablelist

su -m $osuser -c "$ORACLE_HOME/bin/rman target / log=$dbrecover append " << 'EOF'
run {
alter session set "_oracle_script"=true;
ALTER pluggable database all open;
alter pluggable database all save state;
}
exit;
EOF
fi

opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?

if [ $retval -gt 0 ]; then
 oraerror=`grep ORA-01092 $openDBlog`
 logerrormsg EM_DB_READ_WRITE "$oraerror"
 exit 1;
elif [ $opencnt -eq 0 ]; then
 oraerror=`grep ORA-01092 $openDBlog`
 logerrormsg EM_DB_READ_WRITE "$oraerror"
 exit 1;
fi

tempg=0
tempg=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feedback off
select count(*) from dba_tablespace_groups;
exit;
EOF1`

if [ "$tempg" -gt "0" ]; then
tempgn=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feed off;
set echo off;
select distinct group_name from dba_tablespace_groups where group_name in (select property_value from database_properties where property_name = 'DEFAULT_TEMP_TABLESPACE');
exit;
EOF1`
fi

tabgroup="tabgroup_"$nsid".txt"

if [ "x$tempgn" != "x" ]; then
su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1' > $tabgroup
set head off;
set feed off;
set echo off;
select tablespace_name from dba_tablespace_groups where group_name in (select distinct group_name from dba_tablespace_groups where group_name in (select property_value from database_properties where property_name = 'DEFAULT_TEMP_TABLESPACE'));
exit;
EOF1
chmod 755 $tabgroup
else
tempn=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
select property_value FROM DATABASE_PROPERTIES where PROPERTY_NAME='DEFAULT_TEMP_TABLESPACE';
exit;
EOF1`
fi

if [ "$pdbcnt" -eq "0" ]; then
su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $tabfilelist
set head off;
set feed off;
set echo off;
select tb.name tabname from v\$tablespace tb where tb.name != 'TEMP_CL' and ts# not in (select ts# from v\$datafile);
exit;
EOF
else
su -m $osuser -c "sqlplus -S / as sysdba " << EOF > $tabfilelist
set head off;
set feed off;
set echo off;
select tb.name tabname from v\$tablespace tb where tb.name != 'TEMP_CL' and con_id not in (select con_id from v\$pdbs where name != 'PDB\%SEED') and ts# not in (select ts# from v\$datafile where con_id not in (select con_id from v\$pdbs where name != 'PDB\%SEED'));
exit;
EOF
fi

#echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $openDatabase1

echo -e "alter database default temporary tablespace TEMP_CL;" > $openDatabase1

while read line
do
    set -- $line
    tabname=$1
        echo "outside if $tabname"
        if [ "x$tabname" != "x" ]; then
        echo "inside if $tabname"
        tmpstr=" '$dataDir1' size 5g autoextend on;"
        echo "drop tablespace $tabname including contents and datafiles;" >> $openDatabase1
        echo "create temporary tablespace $tabname tempfile $tmpstr" >> $openDatabase1
        fi
done < $tabfilelist


echo -e "shutdown immediate;" >> $openDatabase1
echo -e "startup;" >> $openDatabase1


export ORACLE_BASE=$ASM_BASE
export ORACLE_HOME=$ASMHOME
export LD_LIBRARY_PATH=$ASMHOME/lib:/usr/lib:/lib
export PATH=$ASMHOME/bin:$PATH
export ORACLE_SID="$ASMS"

#reprocess for original SID, to support gc
source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh
#UOSID=`get_o_asm_sid $dataDir $guser`

UOSID=`get_o_asm_sid $dataDir $guser`
if [ -z "$UOSID" ]; then
   logerrormsg EM_MISSING_SUBDIR $dataDir
   #errmsg=`$tdate`$nsid":no backup subdirectory found under $dataDir"
   exit 1
fi

osid=$UOSID

export ORACLE_BASE=
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH
export ORACLE_SID=$nsid

if [ "x$tempgn" != "x" ]; then
while read line
do
    set -- $line
    tabgname=$1
        echo "tab group if $tabgname"
        if [ "x$tabgname" != "x" ]; then
        echo "in tab group if $tabgname"
        echo -e "alter tablespace $tabgname tablespace group $tempgn;" >> $openDatabase1
        fi
done < $tabgroup

echo -e "alter database default temporary tablespace $tempgn;" >> $openDatabase1
else
echo -e "alter database default temporary tablespace $tempn;" >> $openDatabase1
fi

echo -e "create undo tablespace UNDOTBS_CL1 datafile '$dataDir1/$osid/datafile/undotbs_cl1.dbf' size 500m autoextend on maxsize unlimited;" >> $openDatabase1
echo -e "alter system set undo_tablespace=UNDOTBS_CL1;" >> $openDatabase1
echo -e "exit" >> $openDatabase1

change_own $osuser $openDatabase1
change_own $osuser $tabfilelist

export ORACLE_SID=$nsid
su -m $osuser -c "sqlplus / as sysdba @$openDatabase1 " >> $openDBlog
retval=$?

if [ "$nonid" != "true" ]; then
if [ ! -f "$dbfsPath" ]; then
   roTBS='NONE'
else
   roTBS=`cat $dbfsPath`
fi

roCTBS=''
for i in $(echo $roTBS | tr "," "\n")
do
    echo "$i"
    if [ -z "$roCTBS" ]; then
       roCTBS="'''"${i}"'''"
    else
       roCTBS=${roCTBS}"\n'''"${i}"'''"
    fi
done

echo "$roCTBS"

ctbs_input=''
echo -e "$roCTBS" > roCTBS_output.txt
ctbs_input=`cat roCTBS_output.txt`
echo "$ctbs_input"

if [ -e "$runnidsql" ]; then
rm -f $runnidsql
fi

while read line
do
    set -- $line
    readtabname=$1
        if [ "x$readtabname" != "x" ]; then
        echo -e "@nidROTBS.sql $readtabname ONLINE" >> $runnidsql
        echo -e "@nidROTBS.sql $readtabname 'READ WRITE'" >> $runnidsql
        fi
done < roCTBS_output.txt
chmod 755 $runnidsql

su -m $osuser -c "$sqlplus / as sysdba " << EOF
@$runnidsql
shutdown immediate;
startup mount;
exit;
EOF

nidlog="nid_"$nsid".log"
su -m $osuser -c "$ORACLE_HOME/bin/nid target=/ LOGFILE=$nidlog "

if [ ! -z "$DB_TZ" ]; then
echo " ********** setting TZ:$DB_TZ  ***************"
TZ=$DB_TZ
export TZ
fi

if [ ! -f "$runnidsql" ]; then
 rm -f $runnidsql
fi

while read line
do
    set -- $line
    readtabname=$1
        if [ "x$readtabname" != "x" ]; then
        echo -e "@nidROTBS.sql $readtabname 'READ ONLY'" >> $runnidsql
        fi
done < roCTBS_output.txt
chmod 755 $runnidsql

su -m $osuser -c "$sqlplus / as sysdba " << EOF
startup mount;
alter database open resetlogs;
@$runnidsql
exit;
EOF
fi

if [ ! -f "roCTBS_output.txt" ]; then
 rm -f roCTBS_output.txt
fi

export ORACLE_SID=$nsid
opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?

if [ $retval -gt 0 ]; then
 logerrormsg EM_DB_OPEN_POST_UNDO_OP
 exit 1;
elif [ $opencnt -eq 0 ]; then
 logerrormsg EM_DB_OPEN_POST_UNDO_OP
 exit 1;
fi

if [ -e $dbrecover ]; then
   echo "removing $dbrecover "
  #rm -f $dbrecover
fi

if [ -e $recoverDatabase ]; then
   echo "removing $recoverDatabase "
   rm -f $recoverDatabase
fi

if [ -e $openDatabase ]; then
   echo "removing $openDatabase "
   rm -f $openDatabase
fi
if [ -e $openDatabase1 ]; then
   echo "removing $openDatabase1 "
   rm -f $openDatabase1
fi

if [ -e $openPDatabase ]; then
   echo "removing $openPDatabase "
   rm -f $openPDatabase
fi

if [ -e $openTmpDatabase ]; then
   echo "removing $openTmpDatabase "
   rm -f $openTmpDatabase
fi

if [ -e $addUnnamed ]; then
   echo "removing $addUnnamed "
   rm -f $addUnnamed
fi
if [ -e $verifyDatabase ]; then
   echo "removing $verifyDatabase before generating a new one"
   rm -f $verifyDatabase
fi

if [ -e $openDBlog ]; then
   echo "removing $openDBlog "
   #rm -f $openDBlog
fi

if [ -e $tabfilelist ]; then
   echo "removing $tabfilelist "
   rm -f $tabfilelist
fi

nsid1=$nsid
orapw=$ORACLE_HOME"/dbs/orapw"$nsid1

if [ -e $orapw ]; then
   rm -f $orapw
fi

set +x
syspwd="change_on_install"
if [ ! -z $ouser ] && [ "$ouser" == "sys" ] && [ ! -z $opass ]; then
   syspwd=$opass
fi

su -m $osuser -c "$ORACLE_HOME/bin/orapwd file=$orapw password=$syspwd entries=5"
set -x

#grant user sysdba privilege
if [ ! -z $ouser ] && [ "$ouser" != "sys" ]; then
su -m $osuser -c "$sqlplus / as sysdba " << EOF
grant sysdba to $ouser;
exit;
EOF
fi

if [ $pdbcnt -eq 0 ]; then
su -m $osuser -c "$sqlplus / as sysdba " << EOF
shutdown immediate;
startup mount;
alter database archivelog;
alter database open;
exit;
EOF
fi

export ORACLE_SID=$nsid
opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?

if [ $retval -gt 0 ]; then
 logerrormsg EM_DB_OPEN_POST_PWD_FILE_OP
 exit 1;
elif [ $opencnt -eq 0 ]; then
 logerrormsg EM_DB_OPEN_POST_PWD_FILE_OP
 exit 1;
fi

rcount=0
while true
do
{
ponum=0
ponum=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feedback off
select count(*) from dba_rollback_segs where status = 'PARTLY AVAILABLE' or status = 'NEEDS RECOVERY';
exit;
EOF1`

if [ "$ponum" -gt "0" ]; then
undorollofflist=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feed off;
set echo off;
set linesize 4000;
set newpage none;
select '*._offline_rollback_segments='||ltrim(rtrim(listagg(to_char(SEGMENT_NAME),',') WITHIN GROUP (ORDER BY SEGMENT_NAME ))) from dba_rollback_segs where status = 'PARTLY AVAILABLE' or status = 'NEEDS RECOVERY';
exit;
EOF1`

ttbs="*.undo_tablespace=UNDOTBS_CL1"
sed -i '/undo_tablespace/d' $orarollbacktemp
sed -i '/offline_rollback_segments/d' $orarollbacktemp
echo -e $ttbs >> $orarollbacktemp
echo -e $undorollofflist >> $orarollbacktemp
chmod 755 $orarollbacktemp
echo -e "shutdown immediate;" > $openTmpDatabase1
echo -e "startup force restrict pfile='$orarollbacktemp'" >> $openTmpDatabase1
echo -e "alter system set undo_tablespace=UNDOTBS_CL1;" >> $openTmpDatabase1

while read line
do
    set -- $line
    undoname=$1
        echo "outside if $undoname"
        if [ "x$undoname" != "x" ]; then
        echo "inside if $undoname"
        echo "drop tablespace $undoname including contents and datafiles;" >> $openTmpDatabase1
        fi
done < $undofilelist

echo -e "shutdown immediate;" >> $openTmpDatabase1
echo -e "startup;" >> $openTmpDatabase1
echo -e "exit" >> $openTmpDatabase1

chmod 755 $openTmpDatabase1
export ORACLE_SID=$nsid
su -m $osuser -c "sqlplus / as sysdba @$openTmpDatabase1 " >> $openDBlog
retval=$?
rcount=`expr $rcount + 1`
if [ "$rcount" -gt "3" ]; then
 break;
fi
sleep 10;
else
#exit 0;
  break;
fi
}
done


export ORACLE_SID=$nsid

su -m $osuser -c "$sqlplus / as sysdba " << EOF3
drop tablespace TEMP_CL including contents and datafiles;
drop tablespace UNDOTBS_DEF including contents and datafiles;
exit;
EOF3

if [ "$pdbcnt" -gt "0" ]; then
su -m $osuser -c "$sqlplus / as sysdba " << EOF
alter session set "_oracle_script"=true;
ALTER pluggable database all open instances=all;
alter pluggable database all save state instances=all;
exit;
EOF
fi

opencnt=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?

if [ $retval -gt 0 ]; then
 exit 1;
elif [ $opencnt -eq 0 ]; then
 exit 1;
fi

if [ "$retval" -eq "0" ]; then
su -m $osuser -c "$sqlplus / as sysdba " << EOF3
alter database enable block change tracking;
purge recyclebin;
purge dba_recyclebin;
exit;
EOF3
fi

if [ $pdbcnt -gt 0 ]; then
pdbname=`su -m $osuser -c "sqlplus -S / as sysdba " << 'EOF1'
set head off;
set feed off;
set echo off;
set linesize 4000;
set newpage none;
select ltrim(rtrim(listagg(to_char(name),',') WITHIN GROUP (ORDER BY name))) from v\$pdbs where name not like 'PDB%SEED';
exit;
EOF1`

for x in $(echo $pdbname | tr "," "\n")
do
su -m $osuser -c "sqlplus / as sysdba " << EOF
@tmpPDB.sql "$x" "$dataDir1"
exit;
EOF
done
fi

echo -e "set pagesize 100" >> $verifyDatabase
echo -e "set linesize 100" >> $verifyDatabase
echo -e "set echo off" >> $verifyDatabase
echo -e "set feedback off" >> $verifyDatabase
echo -e "set wrap on" >> $verifyDatabase
echo -e "col tablespace_name format a15" >> $verifyDatabase
echo -e "col file_name format a75" >> $verifyDatabase
echo -e "select distinct tablespace_name, file_name from dba_data_files order by 1;" >> $verifyDatabase
echo -e "select owner,count(*) from dba_tables group by owner order by 1;" >> $verifyDatabase
echo -e "select dbid, name, open_mode from v\$database;" >> $verifyDatabase
echo -e "select sum(bytes)/(1024*1024*1024) \"dbSize (GB)\" from dba_data_files;" >> $verifyDatabase
echo -e "select * from v\$instance;" >> $verifyDatabase
echo -e "exit" >> $verifyDatabase

change_own $osuser $verifyDatabase

setdb="setdb_"$nsid
nsid1=$nsid
echo -e "# Modify the following variables based on the environment" > $setdb
echo -e "export ORACLE_SID=$nsid1" >> $setdb
echo -e "export ORACLE_HOME=$ORACLE_HOME" >> $setdb
echo -e " " >> $setdb
echo -e "export PATH=$ORACLE_HOME/bin:$PATH" >> $setdb
echo -e "#END OF FILE" >> $setdb
change_own $osuser $setdb
export ORACLE_SID=$nsid1

otab=$nsid":"$ORACLE_HOME":Y"
echo -e "$otab" >> /etc/oratab
su -m $osuser -c "$sqlplus / as sysdba @$verifyDatabase"
if [ -e $verifyDatabase ]; then
   echo "removing $verifyDatabase before generating a new one"
   rm -f $verifyDatabase
fi

retval=$?
if [ -e $pglist ]; then
   echo "removing $pglist"
   rm -f $pglist
fi
if [ -e $pluggablelist ]; then
   echo "removing $pluggablelist"
   rm -f $pluggablelist
fi
if [ -e $offlinefilelist ]; then
   echo "removing $offlinefilelist"
   rm -f $offlinefilelist
fi
if [ -e $undofilelist ]; then
   echo "removing $undofilelist"
   rm -f $undofilelist
fi
if [ -e $openTmpDatabase1 ]; then
   echo "removing $openTmpDatabase1 "
   rm -f $openTmpDatabase1
fi

exit 0
fi
