#!/bin/sh
#
# ASMOracleRestore.sh: shell script to automate cloning and bring up an Oracle database using backup image copies
#
#usage: ASMOracleRestore.sh <Oracle Home path> <Disk Group Name> <Oracle User> <standalone yes/no> <Grid User> <log mount path> <timestamp yyyymmddhh24mi>"

set -x
oracleHome=$1
backupLocation=$2
ouser=$3
guser=$4
standalone=$5
archmnt=$6
tstamp=$7
dataDir=$backupLocation

if [ "$#" -lt 5 ]; then
echo "This script needs 5 input parameter"
echo "usage: ASMOracleRestore.sh <Oracle Home path> <Disk Group Name> <Oracle User> <Grid User> <standalone yes/no> <log mount path> <timestamp yyyymmddhh24mi>"
echo " For example: ASMOracleRestore.sh /home/oracle/app/oracle/product/11.2.0/dbhome_1 DATA1 oracle grid no /act/mnt/vdbxlog 201404021435"
exit 1
fi

source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh

BASEDIR=$(dirname $0)
echo $BASEDIR
cd $BASEDIR
# correctly set the ORACLE_BASE AND ORACLE_HOME  for your environment
ORACLE_HOME=$oracleHome
export ORACLE_HOME
PATH=$ORACLE_HOME/bin:$PATH
export PATH

cuser=`whoami`

echo "current user is $cuser"

if [ "$cuser" != "$ouser" ]; then
chmod -R 755 $BASEDIR
change_own $ouser $BASEDIR
fi

if [ ! -d $ORACLE_HOME ]; then
   echo "could not find ORACLE_HOME "
   exit 1
fi


backupLocation=`echo $backupLocation | tr '[a-z]' '[A-Z]'`
if [ ! -d /act/touch/$backupLocation ]; then
   backupLocation=`echo $backupLocation | tr '[A-Z]' '[a-z]'`
   if [ ! -d /act/touch/$backupLocation ]; then
        echo "could not find $backupLocation "
        exit 1
   fi
fi
dataDir=$backupLocation

# get the backup parameter file location
mname=$backupLocation
echo $mname
pfileloc=/act/touch/$mname

# get the production sid name from mount
getpname=`ls -ltr $pfileloc | grep '__backup.ora'`
echo $getpname
pname=`echo $getpname | awk '{print $NF}'`
echo $pname
osid=`echo $pname | awk -F"__" '{print $(NF-1)}'`
echo "***************"
echo $osid
nsid=$osid

origfile=$pfileloc/$osid"__backup.ora"
cp $origfile $pfileloc/$osid"__cpfile.ora"
chmod -R 775 $pfileloc/$osid"__cpfile.ora"
orafile=$pfileloc/$osid"__cpfile.ora"
echo $orafile

openDatabase="openDatabase.sql"
openDatabase1="openDatabase1.sql"
openDatabaseTmp="openDatabaseTmp.sql"
recoverDatabase="recoverDatabase.sql"
verifyDatabase="verifyDatabase.sql"
CLONE_CRT_SQL="Clone_createCtrlFileASM.sql"
openDBlog="openDBlog_"$nsid".txt"

sqlplus=$ORACLE_HOME/bin/sqlplus

dbfsPath=/act/touch/$backupLocation/$osid"_datafiles_readonly_catalog.conf"
export dbfsPath
echo "read only tablespace file path : $dbfsPath"

if [ -z $orafile ]; then
   echo "missing init.ora file path"
   exit 1
fi

if [ -e $openDBlog ]; then
#   echo "removing $openDBlog before generating a new one"
   rm -f $openDBlog
fi

echo -e "Opening Database is started..." > $openDBlog

change_own $ouser $openDBlog

var="+ASM"
for ASMSID in $var; do
ASMS=`grep -i "$ASMSID" /etc/oratab | head -n1 | cut -f1 -d":"`
 echo "asmsid is: $ASMS"
ASMHOME=`grep -i "$ASMS:" /etc/oratab | head -n1 | cut -f2 -d":"`
 echo "asmhome is: $ASMHOME"
done

export ORACLE_HOME=$ASMHOME
export LD_LIBRARY_PATH=$ASMHOME/lib:/usr/lib:/lib
export PATH=$ASMHOME/bin:$PATH
export ORACLE_SID="$ASMS"
asmcmd=$ASMHOME/bin/asmcmd

dataDir1="+"$mname
ctlfile=`su -m $guser $asmcmd ls +$mname/$nsid/*.ctl | grep -v _standby`

rctlfile=$dataDir1"/"$nsid"/"$ctlfile

#if [ ! -z "$archmnt" ]; then
#   rctlfile=`ls -ltr $archmnt*/*.* | grep cf-D_ | grep -v _standby. | tail -n 1 | awk '{print $NF}'`  
#fi

if [ "$cuser" != "$ouser" ]; then
suser="su -m $ouser -c"
else
suser=""
fi

su -m $guser -c "$asmcmd << EOF
cd $mname/$nsid
mkdir onlinelog
EOF"

dbrcnt=`grep -i ".db_recovery_file_dest" $orafile | wc -l`

if [ "$dbrcnt" -gt "1" ]; then
dbrecoveryfile="*.db_recovery_file_dest='$dataDir1'"
sed -i '/db_recovery_file_dest=/d' $orafile
echo -e "$dbrecoveryfile" >> $orafile
fi

controlfile="*.control_files='$dataDir1/$nsid/$ctlfile'"
dbcreatefile="*.db_create_file_dest='$dataDir1'"
#dbrecoveryfile="*.db_recovery_file_dest='$dataDir1'"
logarchivedest="*.log_archive_dest_1='LOCATION=$dataDir1/$nsid/archivelog/'"
dbname="*.db_name='$nsid'"
#dbrecoveryfilesize="*.db_recovery_file_dest_size=50g"
processes="*.processes=500"
clustdb="*.cluster_database=false"

sed -i '/control_files=/d' $orafile
sed -i '/db_create_file_dest=/d' $orafile
#sed -i '/db_recovery_file_dest=/d' $orafile
sed -i '/log_archive_dest/d' $orafile
sed -i '/cluster_database=/d' $orafile
sed -i '/db_create_online_log_dest/d' $orafile
#remove undotbs_def
sed -i '/UNDOTBS_DEF/d' $orafile

echo -e "$controlfile" >> $orafile
echo -e "$dbcreatefile" >> $orafile
#echo -e "$dbrecoveryfile" >> $orafile
echo -e "$logarchivedest" >> $orafile
echo -e "$clustdb" >> $orafile

export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH
sqlplus=$ORACLE_HOME/bin/sqlplus

#create sql file to open the database and add tempfile
if [ -e $openDatabase ]; then
   #echo "removing $openDatabase before generating a new one"
   rm -f $openDatabase
fi

if [ -e $openDatabase1 ]; then
   #echo "removing $openDatabase1 before generating a new one"
   rm -f $openDatabase1
fi

if [ -e $openDatabaseTmp ]; then
   #echo "removing $openDatabaseTmp before generating a new one"
   rm -f $openDatabaseTmp
fi

if [ -e $recoverDatabase ]; then
   #echo "removing $recoverDatabase before generating a new one"
   rm -f $recoverDatabase
fi

actdg="ACT"$nsid
# now export new SID, do this outside of shell, before invoke this shell
export ORACLE_SID=$nsid
export ORACLE_HOME=$oracleHome
export PATH=$ORACLE_HOME/bin:$PATH

# copy ora file
initf1=$ORACLE_HOME/dbs/"init"$nsid"1.ora"
initf=$ORACLE_HOME/dbs/"init"$nsid".ora"

# keep a copy of ora file under ORACLE_BASE
#cp $orafile $ORACLE_BASE/"init"$nsid".ora"
# start the database for restore
spf=$dataDir1/$nsid/"spfile"$nsid".ora"

if [ "$standalone" = "no" ]; then
   if [ "x$suser" = "x" ]; then
   srvctl stop database -d $nsid 
   srvctl remove database -d $nsid -f
   echo -e "SPFILE=$spf" > $initf1
   echo -e "SPFILE=$spf" > $initf
   $sqlplus / as sysdba @createPfile.sql $orafile $spf
   if [ $? -ne 0 ]; then
     echo "failure during spfile creation"
     exit 1
   fi
   srvctl add database -d $nsid -o $ORACLE_HOME -a $mname
   srvctl modify database -d $nsid -o $ORACLE_HOME -a $mname
   srvctl modify database -d $nsid -p $spf
   else
   $suser "echo -e SPFILE=$spf > $initf1"
   $suser "echo -e SPFILE=$spf > $initf"
   $suser "$sqlplus / as sysdba @createPfile.sql $orafile $spf "
   $suser "srvctl add database -d $nsid -o $ORACLE_HOME -a $mname"
   $suser "srvctl modify database -d $nsid -o $ORACLE_HOME -a $mname"
   $suser "srvctl modify database -d $nsid -p $spf"
   fi
else
   if [ "x$suser" = "x" ]; then
   srvctl stop database -d $nsid 
   srvctl remove database -d $nsid -f
   echo -e "SPFILE=$spf" > $initf
   $sqlplus / as sysdba @createPfile.sql $orafile $spf
   if [ $? -ne 0 ]; then
     echo "failure during spfile creation"
     exit 1
   fi
   srvctl modify database -d $nsid -o $ORACLE_HOME -a $mname
   srvctl modify database -d $nsid -p $spf
   else
   $suser "echo -e SPFILE=$spf > $initf"
   $suser "$sqlplus / as sysdba @createPfile.sql $orafile $spf "
   $suser "srvctl modify database -d $nsid -o $ORACLE_HOME -a $mname"
   $suser "srvctl modify database -d $nsid -p $spf"   
   fi
fi

dbrecover="dbrecover.txt"
echo "restore controlfile  " > $dbrecover
export dbrecover

if [ "x$suser" = "x" ]; then
echo "changing permission is not required"
else
change_own $ouser $dbrecover
fi

rmanctl="run
{
restore controlfile from '$rctlfile';
alter database mount;
}"

if [ "x$suser" = "x" ]; then
$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
$rmanctl
exit
EOF
else
$suser "$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
$rmanctl
exit
EOF"
if [ $? -ne 0 ]; then
  echo "failure during controlfile restore"
  exit 1
fi
fi

if [ "x$suser" = "x" ]; then
$sqlplus / as sysdba @mountDatabase.sql
if [ $? -ne 0 ]; then
  echo "failure during mount operation"
  exit 1
fi
else
$suser "$sqlplus / as sysdba @mountDatabase.sql"
if [ $? -ne 0 ]; then
  echo "failure during mount operation"
  exit 1
fi
fi

renameRedo="renameRedo.sql"
if [ -e $renameRedo ]; then
   #echo "removing $recoverDatabase before generating a new one"
   rm -f $renameRedo
fi
touch $renameRedo
change_own $ouser $renameRedo
reredo='v\$logfile'

if [ "x$suser" = "x" ]; then
$sqlplus -S / as sysdba << EOF > $renameRedo
SET TERMOUT OFF
SET HEADING OFF
SET PAGESIZE 5000
SET LINESIZE 200
SET TRIMSPOOL OFF
SET WRAP OFF
SET FEEDBACK OFF
SET ECHO OFF

select 'SET TERMOUT OFF' from dual;
select 'SET HEADING OFF' from dual;
select 'SET TRIMSPOOL OFF' from dual;
select 'SET WRAP OFF' from dual;
select 'SET FEEDBACK OFF' from dual;
select 'SET ECHO OFF' from dual;
select 'alter database rename file '||''''||member||''''||' to '||''''||'$dataDir1/$nsid/onlinelog/'||substr(member,instr(member,'/',-1,1)+1)||''''||';' from v\$logfile;
select 'exit;' from dual;
exit;
EOF
change_own $ouser $renameRedo
$sqlplus / as sysdba @renameRedo.sql
if [ $? -ne 0 ]; then
  echo "failure during rename redo operation"
  exit 1
fi
else
$suser "$sqlplus -S / as sysdba << EOF > $renameRedo
SET TERMOUT OFF
SET HEADING OFF
SET PAGESIZE 5000
SET LINESIZE 200
SET TRIMSPOOL OFF
SET WRAP OFF
SET FEEDBACK OFF
SET ECHO OFF

select 'SET TERMOUT OFF' from dual;
select 'SET HEADING OFF' from dual;
select 'SET TRIMSPOOL OFF' from dual;
select 'SET WRAP OFF' from dual;
select 'SET FEEDBACK OFF' from dual;
select 'SET ECHO OFF' from dual;
select 'alter database rename file '||''''||member||''''||' to '||''''||'$dataDir1/$nsid/onlinelog/'||substr(member,instr(member,'/',-1,1)+1)||''''||';' from $reredo;
select 'exit;' from dual;
exit;
EOF"
change_own $ouser $renameRedo
$suser "$sqlplus / as sysdba @renameRedo.sql"
if [ $? -ne 0 ]; then
  echo "failure during rename redo operation"
  exit 1
fi
fi

#dbrecover="dbrecover.txt"

export ORACLE_SID=$nsid
catpath=$dataDir1"/"$nsid

smname=`echo $mname | tr '[A-Z]' '[a-z]'`
switchloc=/act/touch
dbswfile=$switchloc/"."$smname"_switch.conf"

echo "recovering database " >> $dbrecover

#change_own $ouser $dbrecover

echo "******* recovering database *********** "
export dbfsPath
export dbrecover

./recoverRman_tstamp.sh $catpath $nsid $dbswfile $ouser $archmnt $tstamp 

if [ "$cuser" != "$ouser" ]; then
suser="su -m $ouser -c "
else
suser=""
fi

$suser "$sqlplus / as sysdba @newDatafile.sql"
export dbfsPath
export dbrecover
./recoverRman_rotbs.sh $catpath $nsid $dbswfile $ouser $archmnt $tstamp 

$suser "$sqlplus / as sysdba @onlineROfile.sql"

vtabs='v\$tablespace'

export ORACLE_SID=$nsid
if [ "x$suser" = "x" ]; then
temre=`$sqlplus -S / as sysdba << 'EOF'
set head off;
set echo off;
set termout off;
set feedback off;
select count(*) FROM v\$tablespace where name = 'TEMP_RE';
exit;
EOF`
else
temre=`$suser "$sqlplus -S / as sysdba << 'EOF'
set head off;
set echo off;
set termout off;
set feedback off;
select count(*) FROM v\\$tablespace where name = 'TEMP_RE';
exit;
EOF"`
fi

echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $openDatabase
echo -e "alter database open resetlogs;" >> $openDatabase
echo -e "alter database enable block change tracking;" >> $openDatabase
echo -e "exit" >> $openDatabase

export ORACLE_SID=$nsid
if [ "x$suser" = "x" ]; then
$sqlplus / as sysdba @openDatabase.sql >> $openDBlog
else
change_own $ouser $openDatabase
$suser "$sqlplus / as sysdba @openDatabase.sql " >> $openDBlog
retval=$?
if [ $retval -gt 0 ]; then
echo "********** error restoring database ***************"
rm -f $spf
exit 1;
fi
fi

export ORACLE_SID=$nsid
if [ "x$suser" = "x" ]; then
vsort=`$sqlplus -S / as sysdba << 'EOF'
set head off;
set echo off;
set termout off;
set feedback off;
select count(*) FROM v\$sort_usage;
exit;
EOF`
else
vsort=`$suser "$sqlplus -S / as sysdba << 'EOF'
set head off;
set echo off;
set termout off;
set feedback off;
select count(*) FROM v\\$sort_usage;
exit;
EOF"`
fi


if [ "$vsort" -gt "0" ]; then
export ORACLE_SID=$nsid
if [ "x$suser" = "x" ]; then
$sqlplus / as sysdba << 'EOF'
shutdown immediate;
startup;
exit;
EOF
else
$suser "$sqlplus / as sysdba << 'EOF'
shutdown immediate;
startup;
exit;
EOF"
fi
fi

if [ "$temre" -gt "0" ]; then
 echo -e "drop tablespace TEMP_RE including contents and datafiles;" > $openDatabaseTmp
fi
echo -e "CREATE temporary TABLESPACE TEMP_RE TEMPFILE '$dataDir1/$nsid/temp_re01.dbf' size 2g autoextend on maxsize unlimited;" >> $openDatabaseTmp
echo -e "exit" >> $openDatabaseTmp

export ORACLE_SID=$nsid
if [ "x$suser" = "x" ]; then
$sqlplus / as sysdba @openDatabaseTmp.sql >> $openDBlog
else
change_own $ouser $openDatabaseTmp
$suser "$sqlplus / as sysdba @openDatabaseTmp.sql " >> $openDBlog
retval=$?
if [ $retval -gt 0 ]; then
echo "********** error restoring database ***************"
rm -f $spf
exit 1;
fi
fi

if [ "$standalone" = "no" ]; then
if [ "x$suser" = "x" ]; then
tempn=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
set echo off;
set termout off;
set feedback off;
select property_value FROM DATABASE_PROPERTIES where PROPERTY_NAME='DEFAULT_TEMP_TABLESPACE';
exit;
EOF1`
else
tempn=`$suser "$sqlplus -S / as sysdba << 'EOF1'
set head off;
set echo off;
set termout off;
set feedback off;
select property_value FROM DATABASE_PROPERTIES where PROPERTY_NAME='DEFAULT_TEMP_TABLESPACE';
exit;
EOF1"`
fi

echo "$tempn " | sed -e 's/^ *//' -e 's/ *$//'
tempn=`echo $tempn | tr '\n' ' '`

export ORACLE_SID=$nsid
if [ "x$suser" = "x" ]; then
vsort=`$sqlplus -S / as sysdba << 'EOF'
set head off;
set echo off;
set termout off;
set feedback off;
select count(*) FROM v\$sort_usage;
exit;
EOF`
else
vsort=`$suser "$sqlplus -S / as sysdba << 'EOF'
set head off;
set echo off;
set termout off;
set feedback off;
select count(*) FROM v\\$sort_usage;
exit;
EOF"`
fi

#echo "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $openDatabase1
echo -e "alter database default temporary tablespace TEMP_RE;" > $openDatabase1
if [ "$vsort" -gt "0" ]; then
echo -e "shutdown immediate;" >> $openDatabase1
echo -e "startup;" >> $openDatabase1
fi
echo -e "drop tablespace $tempn including contents and datafiles;" >> $openDatabase1
echo -e "CREATE temporary TABLESPACE $tempn TEMPFILE '$dataDir1/$nsid/TEMP1_101.dbf' size 5g autoextend on maxsize unlimited;" >> $openDatabase1
echo -e  "alter database default temporary tablespace $tempn;" >> $openDatabase1
if [ "$standalone" = "no" ]; then
   echo -e  "alter system set cluster_database=TRUE scope=spfile sid='*';" >> $openDatabase1
fi
echo -e "shutdown immediate;" >> $openDatabase1
echo -e  "exit" >> $openDatabase1

export ORACLE_SID=$nsid
if [ "x$suser" = "x" ]; then
$sqlplus / as sysdba @openDatabase1.sql >> $openDBlog
else
chmod  777 $openDatabase1
$suser "$sqlplus / as sysdba @openDatabase1.sql " >> $openDBlog
fi
fi #end of check standalone if

if [ -e $recoverDatabase ]; then
   echo " $recoverDatabase "
#  rm -f $recoverDatabase
fi

if [ -e $openDatabase ]; then
   echo "removing $openDatabase "
#   rm -f $openDatabase
fi

if [ -e $openDatabase1 ]; then
   echo "removing $openDatabase1 "
#   rm -f $openDatabase1
fi

if [ "$standalone" = "no" ]; then
# Copy pfile to other nodes
./copyPfileToNodes.sh $oracleHome $nsid $BASEDIR $ouser $dbswfile

  if [ "x$suser" = "x" ]; then
     srvctl start database -d $nsid
  else
    $suser "srvctl start database -d $nsid"
  fi
else
  export ORACLE_SID=$nsid
  if [ "x$suser" = "x" ]; then
   $sqlplus / as sysdba << 'EOF'
   startup;
   exit;
EOF
   else
   $suser "$sqlplus / as sysdba << 'EOF'
   startup;
   exit;
EOF"
   fi 
fi

if [ $retval -gt 0 ]; then
echo "********** error restoring database ***************"
exit 1;
fi

if [ "$standalone" = "no" ]; then
nsid1=`ps -ef | grep pmon | grep -i $nsid | awk '{ print $NF }' | cut -f3 -d"_"`
ORACLE_SID=$nsid1
else
ORACLE_SID=$nsid
fi

export ORACLE_SID

if [ "x$suser" = "x" ]; then
$sqlplus / as sysdba << EOF3
drop tablespace TEMP_RE including contents and datafiles;
exit;
EOF3
else
$suser "$sqlplus / as sysdba << EOF3
drop tablespace TEMP_RE including contents and datafiles;
exit;
EOF3"
fi

rcmd=$dataDir1/$nsid/controlfile/snapcf_$nsid.f
rmancmd="run
{
CONFIGURE SNAPSHOT CONTROLFILE NAME TO '$rcmd';
}"

if [ "x$suser" = "x" ]; then
$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
$rmancmd
exit
EOF
else
$suser "$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
$rmancmd
exit
EOF"
fi

if [ -e $dbrecover ]; then
    echo " $dbrecover "
   #rm -f $dbrecover
fi

exit 0

