#!/bin/sh

# asmdgRebalance.sh: shell script to rebalance the ASM diskgroup
#
# usage: asmdgRebalance.sh <Disk group Name> <Oracle user>  <Grid User>"
set +x
dgname=$1
ouser=$2
guser=$3

if [ "$#" -ne 3 ]; then
 echo "This script needs 3 input parameter"
 echo "usage: asmdgRebalance.sh  <Disk group Name> <Oracle user> <Grid User>"
 echo " For example: asmdgRebalance.sh DATA1 oracle grid"
exit 1
fi

BASEDIR=$(dirname $0)
echo $BASEDIR
cd $BASEDIR

cuser=`whoami`

if [ "$cuser" = "$ouser" ]; then
suser=""
else if [ "$cuser" = "$guser" ]; then
suser=""
else
suser="su -m $guser -c "
fi
fi

dgname=`echo $dgname | tr '[a-z]' '[A-Z]'`
if [ ! -d /act/touch/$dgname ]; then
   dgname=`echo $dgname | tr '[A-Z]' '[a-z]'`
   if [ ! -d /act/touch/$dgname ]; then
        echo "could not find $dgname "
        exit 1
   fi
fi

var="+ASM"
for ASMSID in $var; do
ASMS=`grep -i "$ASMSID" /etc/oratab | head -n1 | cut -f1 -d":"`
 echo "asmsid is: $ASMS"
ASMHOME=`grep -i "$ASMS:" /etc/oratab | head -n1 | cut -f2 -d":"`
 echo "asmhome is: $ASMHOME"
done

export ORACLE_HOME=$ASMHOME
export LD_LIBRARY_PATH=$ASMHOME/lib:/usr/lib:/lib
export PATH=$ASMHOME/bin:$PATH
export ORACLE_SID="$ASMS"
asmcmd=$ASMHOME/bin/asmcmd

adlist="";
ddlist="";

while read line;
 do
 dname=$line;
 if [ -z $adlist ]; 
  then 
  adlist=${adlist}"'$dname'";
 else
  adlist=${adlist}",\n""'$dname'";
 fi;
done < $BASEDIR/asm_disks.conf

echo -e "$adlist"
dgname=`echo $dgname | tr '[a-z]' '[A-Z]'`
dgname1="'$dgname'"

vasmdisk='v\$asm_disk'
vasmgrp='v\$asm_diskgroup'

if [ "x$suser" = "x" ]; then
dd_list=`sqlplus -S / as sysasm << EOF
set head off;
set echo off;
set termout off;
set feedback off;
select name from v\\$asm_disk where group_number in (select group_number from v\\$asm_diskgroup where name=$dgname1);
exit;
EOF`
else
dd_list=`$suser "sqlplus -S / as sysasm << EOF
set head off;
set echo off;
set termout off;
set feedback off;
select name from $vasmdisk where group_number in (select group_number from $vasmgrp where name=$dgname1);
exit;
EOF"`
fi

echo -e "$dd_list"
echo -e "$dd_list " | sed -e 's/^ *//' -e 's/ *$//'
dd_list=`echo $dd_list | tr '\n' ' '`
echo -e "$dd_list"

shopt -s extglob

for f in $dd_list
do
f="${f##*( )}"
f="${f%%*( )}"

if [ -z $ddlist ]; then
	ddlist=${ddlist}"'$f'";
else
	ddlist=${ddlist}",\n""'$f'";
fi
done

shopt -u extglob

echo -e "$ddlist"

doRebalance="doRebalance.sql"

if [ -e $doRebalance ]; then
   rm -f $doRebalance
fi

echo -e "alter diskgroup $dgname add disk " > $doRebalance
echo -e "$adlist" >> $doRebalance
echo -e " drop disk " >> $doRebalance
echo -e "$ddlist" >> $doRebalance
echo -e " rebalance power 11;" >> $doRebalance

if [ "x$suser" = "x" ]; then
sqlplus / as sysasm @doRebalance.sql << EOF
exit;
EOF
else
chmod 777 $doRebalance
su -m $guser -c "sqlplus / as sysasm @doRebalance.sql << EOF
exit;
EOF"
fi

exit 0
