#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# ==============================================================================#
# This script called from the actifio connector for application consistent backups. AIX & LINUX only.
#
#
#set -x
this_script_dir=$(cd `dirname $0` && pwd)
this_script=$(basename $0)
LOG_FILE_NAME=db2backup
func=$1
func=`echo $func | awk '{print tolower($1)}'`
h=`hostname`
flagprefix="/tmp/db2_actifio"
script_log_dir="/act/scripts/log"
TS1=$(date +'%C%y%m%d')
dblog="${script_log_dir}/${LOG_FILE_NAME}.$TS1.log"
dblog_retn=7
logretention=30
max_simul_db_call_ctr=10
max_simul_inst_call_ctr=5

if [ ! -d $script_log_dir ]; then
   echo "$(date +"%Y-%m-%d %H:%M:%S") - creating $script_log_dir directory..." >> $dblog
   mkdir $script_log_dir
   chmod 777 $script_log_dir
fi

if [ ! -d $script_log_dir ]; then
   echo "$(date +"%Y-%m-%d %H:%M:%S") - could not find $script_log_dir script folder. exiting ...."
   echo "$(date +"%Y-%m-%d %H:%M:%S") - could not find $script_log_dir script folder. exiting ...." >> $dblog
   exit 8
fi

if [ ! -f $dblog ]; then
   echo "$(date +"%Y-%m-%d %H:%M:%S") - creating dblog file..."
   echo " " > $dblog
   chmod 777 $dblog
fi

# remove all flagfiles from a previous run
flag_cleanup() {
  echo "--begin: $func section @ $(date +"%Y-%m-%d %H:%M:%S")-----------------------------------------------------------------------" >> $dblog
  echo "$(date +"%Y-%m-%d %H:%M:%S") - DB2 $this_script $func" >> $dblog
  flag=$flagprefix"*"
  count=`ls $flag 2> /dev/null | wc | awk '{print $1}'`
  if [[ "$count" -gt 0 ]]; then
    echo "$(date +"%Y-%m-%d %H:%M:%S") - Found "$count" flag files from previous run - now cleaning" >> $dblog
    rm -f $flag
  fi
}

# To control maximum simultaneous instance script calls thru a set limit, from within the main script
max_simul_inst_call_cntrl() {
  inst_scr_pids="$! $inst_scr_pids"
  inst_call_load_ctr=`echo "$inst_scr_pids" | awk '{print NF}'`
  while [[ $inst_call_load_ctr -eq $max_simul_inst_call_ctr ]]
   do
   ## echo "presently $inst_call_load_ctr simultaneous $this_script instance script call processes are active...." >> $dblog
     for INST_PID in $inst_scr_pids
     do
       ps -p ${INST_PID} 2>&1 >/dev/null
       INST_PS_RC=$?
       if [[ $INST_PS_RC -ne 0 ]]
       then
          inst_scr_pids=`echo "$inst_scr_pids" | sed -e "s/${INST_PID}//g"`
          inst_scr_pids=`echo "$inst_scr_pids" | tr -s " "`
          inst_call_load_ctr=`echo "$inst_scr_pids" | awk '{print NF}'`
          if [[ $inst_call_load_ctr -lt $max_simul_inst_call_ctr ]];then
             break
          fi
       fi
     done
   done
}

# To make one last call to control maximum simultaneous instance script calls thru a set limit, from within the main script
last_simul_inst_call_cntrl() {
  ## /usr/bin/wait 2>/dev/null
  for INST_PID in $inst_scr_pids
  do
    ps -p ${INST_PID} 2>&1 >/dev/null
    INST_PS_RC=$?
    if [ "$INST_PS_RC" == "0" ]
    then
       while [ "$INST_PS_RC" == "0" ]
       do
         sleep 1
         ps -p ${INST_PID} 2>&1 >/dev/null
         INST_PS_RC=$?
       done
    fi
  done
  sleep 5
  ## echo "All simultaneous $this_script instance script call processes initiated from within the main script has completed now." >> $dblog
}

# To control maximum simultaneous database script calls thru a set limit, from within the main script
max_simul_db_call_cntrl() {
  db_scr_pids="$! $db_scr_pids"
  db_call_load_ctr=`echo "$db_scr_pids" | awk '{print NF}'`
  while [[ $db_call_load_ctr -eq $max_simul_db_call_ctr ]]
   do
   ## echo "presently $db_call_load_ctr simultaneous $this_script database script call processes are active...." >> $dblog
     for DB_PID in $db_scr_pids
     do
       ps -p ${DB_PID} 2>&1 >/dev/null
       DB_PS_RC=$?
       if [[ $DB_PS_RC -ne 0 ]]
       then
          db_scr_pids=`echo "$db_scr_pids" | sed -e "s/${DB_PID}//g"`
          db_scr_pids=`echo "$db_scr_pids" | tr -s " "`
          db_call_load_ctr=`echo "$db_scr_pids" | awk '{print NF}'`
          if [[ $db_call_load_ctr -lt $max_simul_db_call_ctr ]];then
             break
          fi
       fi
     done
   done
}

# To make one last call to control maximum simultaneous database script calls thru a set limit, from within the main script
last_simul_db_call_cntrl() {
  ## /usr/bin/wait 2>/dev/null
  for DB_PID in $db_scr_pids
  do
    ps -p ${DB_PID} 2>&1 >/dev/null
    DB_PS_RC=$?
    if [ "$DB_PS_RC" == "0" ]
    then
       while [ "$DB_PS_RC" == "0" ]
       do
         sleep 1
         ps -p ${DB_PID} 2>&1 >/dev/null
         DB_PS_RC=$?
       done
    fi
  done
  sleep 10
  ## echo "All simultaneous $this_script database script call processes initiated from within the main script has completed now." >> $dblog
}

# called directly from main loop with one parm: func $1 to enumerate instances and spawn script async to enum dbs
call_inst() {
  func=$1
  echo "$(date +"%Y-%m-%d %H:%M:%S") - DB2 $this_script $func - capture running instances" >> $dblog
  db2i=`ps -ef | grep -e db2sysc | grep -v -e grep | awk '{print $1}' | sort -u`
  echo "$(date +"%Y-%m-%d %H:%M:%S") - Running db2sysc instances: "$db2i >> $dblog
  for inst in $db2i; do
    echo "$(date +"%Y-%m-%d %H:%M:%S") - Call: $this_script $func $inst &" >> $dblog
    nohup $this_script_dir/$this_script $func $inst >> $dblog 2>&1 &
    max_simul_inst_call_cntrl
  done
  last_simul_inst_call_cntrl
}


# call from main loop with two parms: func $1 to enum dbs for inst $3 and make a further async spawn for each db
call_db() {
  inst=$2
  echo "$(date +"%Y-%m-%d %H:%M:%S") - $this_script $func $inst - indirect (local) databases" >> $dblog
  #echo "set db2home "$inst >> $dblog
  db2home=`cat /etc/passwd | grep ${inst}: |awk -F: '{print $6}'`
  #echo "set db2profile "$db2home >> $dblog
  . $db2home"/sqllib/db2profile"
  local_db=`db2 list db directory | sed -e '/./{H;$!d;}' -e 'x;/Indirect/!d' | grep alias | awk '{print $NF}' | sort`
  echo "$(date +"%Y-%m-%d %H:%M:%S") - Local DB: $inst: $local_db" >> $dblog
  for DB in $local_db; do
    flagfile=$flagprefix"_"$inst"_"$DB
    echo "$(date +"%Y-%m-%d %H:%M:%S") - Create flag file: $flagfile" >> $dblog
    touch $flagfile
    chmod 666 $flagfile
    nohup $this_script_dir/$this_script $1 $inst $DB >> $dblog 2>&1 &
    max_simul_db_call_cntrl
  done
  last_simul_db_call_cntrl
}


# call from main loop with three parms: perform func $1 against inst $2 db $3
call_sub() {
  func=$1
  inst=$2
  DB=$3
  echo "$(date +"%Y-%m-%d %H:%M:%S") - $this_script $func $inst $DB" >> $dblog
  db2home=`cat /etc/passwd | grep ${inst}: | awk -F: '{print $6}'`
  . $db2home"/sqllib/db2profile"
  db2bin=$db2home"/sqllib/bin"
  hadr_db_role=`db2 get db cfg for $DB | awk '/HADR database role/{print toupper($5)}'`
  if [[ "$func" == "freeze" ]];then
     db_part_nums=`awk '{print $1}' $db2home/sqllib/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | sort -nr`
     if [[ "$hadr_db_role" == "STANDBY" ]];then
        db2 deactivate db $DB >> $dblog
     else
        db2 deactivate db $DB >> $dblog
        for DB_PART_NUM in $db_part_nums; do
            echo "$(date +"%Y-%m-%d %H:%M:%S") - set write SUSPEND - DB2NODE: $DB_PART_NUM" >> $dblog
            export DB2NODE=$DB_PART_NUM
            db2 terminate >> $dblog
            db2 connect to $DB >> $dblog
            db2 set write suspend for database >> $dblog
            db2 connect reset >> $dblog
        done
     fi
  fi
  if [[ "$func" == "thaw" ]];then
     db_part_nums=`awk '{print $1}' $db2home/sqllib/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | sort -n`
     if [[ "$hadr_db_role" == "STANDBY" ]];then
        db2 activate db $DB >> $dblog
     else
        for DB_PART_NUM in $db_part_nums; do
            echo "$(date +"%Y-%m-%d %H:%M:%S") - set write RESUME - DB2NODE: $DB_PART_NUM" >> $dblog
            export DB2NODE=$DB_PART_NUM
            db2 terminate >> $dblog
            db2 connect to $DB >> $dblog
            db2 set write resume for database >> $dblog
            db2 connect reset >> $dblog
        done
        db2 activate db $DB >> $dblog
     fi
  fi
  if [[ "$func" == "prune" ]];then
     num_db_part=`awk '{print $1}' $db2home/sqllib/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | wc -l`
     awk '{print $1" "$2}' $db2home/sqllib/db2nodes.cfg | awk '{sub(/^[ \t]+/, "")};1' | grep -v "^#" | while read DB_PART_NUM_NODE
     do
       db_part_nums=`echo $DB_PART_NUM_NODE | awk '{print $1}'`
       db_part_node=`echo $DB_PART_NUM_NODE | awk '{print $2}'`
       echo "$(date +"%Y-%m-%d %H:%M:%S") - PURGE archive log files - DB2NODE: $db_part_nums - $db_part_node" >> $dblog
       export DB2NODE=$db_part_nums
       db2 terminate >> $dblog
       db2 connect to $DB >> $dblog
       log_arch_meth1=`db2 get db cfg for $DB | awk '/LOGARCHMETH1/{print toupper($7)}'`
       log_arch_meth2=`db2 get db cfg for $DB | awk '/LOGARCHMETH2/{print toupper($7)}'`
       db2 connect reset >> $dblog

       # check for db2 $inst - $DB, if in circular log mode
       [[ "$log_arch_meth1" == "OFF" ]] && [[ "$log_arch_meth2" == "OFF" ]] && break
       [[ "$log_arch_meth1" == "NO" ]] && [[ "$log_arch_meth2" == "NO" ]] && break

       log_arch_opt1=`echo $log_arch_meth1 | cut -f1 -d: | awk '{print toupper($1)}'`
       if [[ "$log_arch_opt1" == "DISK" ]]; then
          log_arch_path1=`echo $log_arch_meth1 | cut -f2- -d:`
          echo "Removing $inst - $DB archive log files older than $logretention days from $log_arch_path1" >> $dblog
          if [[ "$num_db_part" -gt 1 ]]; then
             rsh $db_part_node "find ${log_arch_path1} -name S\*\.LOG -type f -mtime +$logretention -print -exec rm -f {} \;" >> $dblog
          else
             find ${log_arch_path1} -name S\*\.LOG -type f -mtime +$logretention -print -exec rm -f {} \; >> $dblog
          fi
       fi
     done
  fi
  if [[ "$func" == "test" ]];then
    echo "$(date +"%Y-%m-%d %H:%M:%S") - $func $inst $DB" >> $dblog
  fi
  flagfile=$flagprefix"_"$inst"_"$DB
  rm -f $flagfile
  echo "$(date +"%Y-%m-%d %H:%M:%S") - Removed flag file: $flagfile" >> $dblog
  return 0
}


# Check database func background tasks completion, before exiting, based on flagfiles count remaining
flag_check() {
  i=0
  f_complete=0
  sleep 5
  flag=$flagprefix"*"
  while [ $f_complete != "1" ]; do
    count=`ls $flag 2> /dev/null | wc | awk '{print $1}'`
    echo "$(date +"%Y-%m-%d %H:%M:%S") -Check background tasks have completed before exiting: $count remaining." >> $dblog
    if [[ "$count" == "0" ]]; then
      f_complete=1
    fi
    sleep 2
    let i=i+1
    if [[ "$i" -gt 15 ]]; then
      echo "$(date +"%Y-%m-%d %H:%M:%S") Could not complete in sufficient time - Aborting main loop" >> $dblog
      exit 8
    fi
  done
  echo "$(date +"%Y-%m-%d %H:%M:%S") DB2 $this_script completed main loop" >> $dblog
  echo "--end:   $func section @ $(date +"%Y-%m-%d %H:%M:%S")-----------------------------------------------------------------------" >> $dblog

}


#echo "\$0 : "$0 >> $dblog
#echo "\$1 : "$1 >> $dblog
#echo "\$2 : "$2 >> $dblog
#echo "\$3 : "$3 >> $dblog


  if [[ "$func" == "abort" ]] || [[ "$func" == "resume" ]]; then
    func="thaw"
  fi

  if [[ "$func" == "suspend" ]]; then
    func="freeze"
  fi

  if [[ "$func" == "purge" ]] || [[ "$func" == "prune" ]]; then
    func="prune"
  fi

  if [[ "$func" == "test" ]] || [[ "$func" == "prune" ]] || [[ "$func" == "freeze" ]] || [[ "$func" == "thaw" ]]; then
    if [[ "$2" != "" ]]; then
      if [[ "$3" != "" ]]; then
        call_sub $func $2 $3       # call to perform function $func for specific instance $2 db $3
      else
        call_db $func $2           # call to enum databases for each instance $2
      fi
    else
      flag_cleanup                 # call flag_cleanup from outer shell, (not if multiple/invalid parms)
      call_inst $func              # outer call to enum instances
      flag_check                   # call flag_check from outer shell, (not if multiple/invalid parms)
    fi

    # Actifio Log file cleanup

    echo "$(date +"%Y-%m-%d %H:%M:%S") - cleaning up old log files: " >> $dblog
    find ${script_log_dir} -name "${LOG_FILE_NAME}*.log" -type f -mtime +$dblog_retn -print -exec rm -f {} \; >> $dblog

  else

    echo "$(date +"%Y-%m-%d %H:%M:%S") - Unrecognised parameter: "$func" - exiting" >> $dblog
    exit 8
  fi

exit 0
