#!/bin/bash

set -x

#export ACT_JOBTYPE="mount"
#export ACT_PHASE="post"

echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ $ACT_JOBTYPE = "unmount" ] || [ $ACT_MULTI_OPNAME = "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ $ACT_JOBTYPE = "mount" ] || [ $ACT_MULTI_OPNAME = "mount" ] ; then
   if [ $ACT_PHASE = "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ "$ACT_LOGSMART_TYPE" = "log" ] ; then
        echo "** No op for log during unmount **"
     exit
   fi
   if [ $ACT_PHASE = "post" ] ; then
        echo "************************* start post during mount ************************"

. /act/scripts/act_Conf_Target_Primary.conf

if [ -z $TARGET_PRIMARY_NODE ]; then
   echo "ERROR TARGET_PRIMARY_NODE IP is not set , check /act/scripts/act_Conf_Target_Primary.conf file existing..."
   exit 1
fi

if [ -z $TARGET_PRIMARY_PORT ]; then
   echo "ERROR TARGET_PRIMARY_PORT mongdb port is not set, check /act/scripts/act_Conf_Target_Primary.conf file existing..."
   exit 1
fi

if [ -z $MONGO_REPLICASET ]; then
   echo "ERROR MONGO_REPLICASET target mongdb replicaset name is not set, check /act/scripts/act_Conf_Target_Primary.conf file existing..."
   exit 1
fi

if [ -z $TARGET_PRIMARY_MOUNTPOINT ]; then
   echo "ERROR TARGET_PRIMARY_MOUNTPOINT target mount point of image is not set, check /act/scripts/act_Conf_Target_Primary.conf file existing..."
   exit 1
fi

TARGET_PRIMARY_NODE=$TARGET_PRIMARY_NODE
TARGET_PRIMARY_PORT=$TARGET_PRIMARY_PORT
TARGET_PRIMARY_MOUNTPOINT=$TARGET_PRIMARY_MOUNTPOINT
MONGO_REPLICASET=$MONGO_REPLICASET

echo "*************** target mountpoint is $TARGET_PRIMARY_MOUNTPOINT ******************"

cmd="mongod --dbpath $TARGET_PRIMARY_MOUNTPOINT --port $TARGET_PRIMARY_PORT"
echo $cmd
nohup $cmd > $TARGET_PRIMARY_MOUNTPOINT/logfile.out 2>&1 &

sleep 10
echo "removing old replica config"
mongo --port $TARGET_PRIMARY_PORT << EOF
use local
db.dropDatabase();
exit
EOF

echo "stopping mongo instance"
pmonid=`ps aux | grep -i mongod | grep -i dbpath | grep -i $TARGET_PRIMARY_PORT | awk {'print $2'}`

if [ ! -z $pmonid ] && [ "$pmonid" -gt "0" ]; then
echo "stopping mongo process id $pmonid running on port $TARGET_PRIMARY_PORT"
kill -9 $pmonid
fi
sleep 5
echo "Re-starting mongo instance with replica set"

cmd="mongod --dbpath $TARGET_PRIMARY_MOUNTPOINT --port $TARGET_PRIMARY_PORT --replSet $MONGO_REPLICASET"
echo $cmd
nohup $cmd > $TARGET_PRIMARY_MOUNTPOINT/logfile.out 2>&1 &
sleep 10
 
echo "Adding new config and replicaset"
mongo --port $TARGET_PRIMARY_PORT << EOF
use local
rs.initiate( {
   _id : "$MONGO_REPLICASET",
   members: [ { _id : 0, host : "$TARGET_PRIMARY_NODE:$TARGET_PRIMARY_PORT" } ]
})
exit
EOF

sleep 10

echo "checking the mongo instances"
mongo --port $TARGET_PRIMARY_PORT << EOF
rs.status()
db.test.count()
exit
EOF
echo "connected to mongo instance running on $TARGET_PRIMARY_PORT"

fi
fi

exit

