#!/bin/bash
TIMESTAMP=`date +%F-%H%M`

# exit if something fails
set -e

if [ $# -lt 3 ]
then
  echo "Usage: act_mongo_recovery.sh <mongo host ip/name> <mongo port> <mongo log mountpoint> <mongo recovery stamp value> "
  echo "Example: ip-172-20-44-49.actifio.com 27018 /mongologbackup 1510865999 "
  exit 1
fi

# get instance address
restoretype="oplog"

# get the dump location
MONGO_HOST=$1
MONGO_PORT=$2
DUMP_LOCATION=$3
echoval=$4

DBHOST="$MONGO_HOST"
DBLOGFILE=$DUMP_LOCATION/$DBHOST-$(date +%H%M).log       # Logfile Name
DBLOGERR=$DUMP_LOCATION/ERRORS_$DBHOST-$(date +%H%M).log # Logfile Name

OPLOGFILE=$DUMP_LOCATION/$DBHOST-$(date +%H%M).log       # Logfile Name
OPLOGERR=$DUMP_LOCATION/ERRORS_$DBHOST-$(date +%H%M).log # Logfile Name

touch "$DBLOGFILE"
exec 6>&1           # Link file descriptor #6 with stdout.
                    # Saves stdout.
exec > "$DBLOGFILE"     # stdout replaced with file $DBLOGFILE.

touch "$DBLOGERR"
exec 7>&2           # Link file descriptor #7 with stderr.
                    # Saves stderr.
exec 2> "$DBLOGERR"     # stderr replaced with file $OPLOGERR.

touch "$OPLOGFILE"
exec 6>&1           # Link file descriptor #6 with stdout.
                    # Saves stdout.
exec > "$OPLOGFILE"     # stdout replaced with file $OPLOGFILE.

touch "$OPLOGERR"
exec 7>&2           # Link file descriptor #7 with stderr.
                    # Saves stderr.
exec 2> "$OPLOGERR"     # stderr replaced with file $OPLOGERR.


  for filename in `find $DUMP_LOCATION/ -name "*.tgz"`
   do
     echostamp=$(ls -t $filename | cut -d '.' -f2)
      if [ ! -z "$echoval" ]; then
          if [ "$echostamp" -le "$echoval" ]; then
             tfile=`ls $DUMP_LOCATION/*.tgz | grep $echostamp`
          if [ -z "$tfile" ]; then
             tfile=`ls $DUMP_LOCATION/ | head -n1`
          fi
        cd $DUMP_LOCATION
        tar -xzf $tfile
        tdfile=`ls -d $DUMP_LOCATION/*/ | grep $echostamp`
        mv $tdfile/local/oplog.rs.bson $tdfile/local/oplog.bson
        mongorestore -h $MONGO_HOST:$MONGO_PORT --oplogReplay $tdfile/local
        MRESTSTATUS=$? 
        if [ $MRESTSTATUS -ne 0 ]; then
        echo "ERROR: mongorestore failed: $DBLOGERR" >&2
        else
        echo "mongorestore completed successfully"
        fi
     fi
    else
                   tfile=`ls $DUMP_LOCATION/*.tgz | grep $echostamp`
          if [ -z "$tfile" ]; then
             tfile=`ls $DUMP_LOCATION/ | head -n1`
          fi
        cd $DUMP_LOCATION
        tar -xzf $tfile
        tdfile=`ls -d $DUMP_LOCATION/*/ | grep $echostamp`
        mv $tdfile/local/oplog.rs.bson $tdfile/local/oplog.bson
        mongorestore -h $MONGO_HOST:$MONGO_PORT --oplogReplay $tdfile/local
        MRESTSTATUS=$?
        if [ $MRESTSTATUS -ne 0 ]; then
        echo "ERROR: mongorestore failed: $DBLOGERR" >&2
        else
        echo "mongorestore completed successfully"
        fi
    fi
  done
exit 

