#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# Description: 
#
# ==============================================================================

ImageId=$1

if [ "X$ImageId" = "X" ]; then
    echo "Usage: $0 <Image_id>"
    exit 1
fi

JobId=$(echo $ImageId | sed 's/Image/Job/')
LogFile="/tmp/"$JobId

grep $JobId /var/act/log/UDSAgent* > $LogFile
if [ ! -s "$LogFile" ]; then
    echo "Cannot find the job $JobId in UDSAgent log"
    rm -f $LogFile 2>/dev/null
    exit 1
fi

mntptfile="/var/act/log/"$ImageId
if [ -f "$mntptfile" ]; then
   rm -f $mntptfile
fi
touch $mntptfile

appnames=`grep "Received object mappings: Backed-up" $LogFile 2>/dev/null | awk '{print $11}'`
lunIds=`grep "Received object mappings: Backed-up" $LogFile 2>/dev/null | awk '{print $15}'`
index=0
for i in $appnames; do
    appname[$index]=$i
    index=`expr $index + 1`
done
index=0
for i in $lunIds; do
    lunId[$index]=$i
    index=`expr $index + 1`
done

i=0
key=0
app_name=
while [ $i -lt $index ]; do
    if [ "X$app_name" = "X${appname[$i]}" ]; then
        appluns[$key]="${appluns[$key]} ${lunId[$i]}"
        i=`expr $i + 1`
        continue
    fi
    key=`expr $key + 1`
    appluns[$key]="${appname[$i]} ${lunId[$i]}"
    app_name=${appname[$i]}
    i=`expr $i + 1`
done
count=$key

#
# log message "Mountedmap bvolguid=/dev/sdb bvolroot=prdgdb bvgname= mvolguid=/dev/sdg mvolroot=/child_mnt mvolmntpts= mvgname="
#
LogData=`grep "Mountedmap bvolguid" $LogFile 2>/dev/null | awk '{for(i=5;i<=NF;i++) {print $i}}'`
if [ "X$LogData" = "X" ]; then
    echo "Cannot find mounted map details in the UDSAgent log"
    rm -f $LogFile 2>/dev/null
    exit 1
fi

index=0
lunid=
for data in $LogData; do
    name=`echo $data | awk -F '=' '{print $1}'`
    value=`echo $data | awk -F '=' '{print $2}'`

    if [ "X$name" = "Xbvolroot" ]; then
        key=1
        while [ $key -le $count ]; do
            set -- ${appluns[$key]}
            sapp=$1
            if [ "X$sapp" = "X$value" ]; then
                lunid=$2
                echo "Appname and Mapped Lun ID(s) ${appluns[$key]}"
                break
            fi
            key=`expr $key + 1`
        done
    elif [ "X$name" = "Xmvolguid" ]; then
        index=`expr $index + 1`
        mountedvolumeid[$index]=$value
        echo "Mounted Volume ID - ${mountedvolumeid[$index]}"
        if [ "X$lunid" != "X" ]; then
            mountedlunId[$index]=$lunid
            if [ "$#" -gt 2 ]; then
                set -- ${appluns[$key]}
                shift
                shift
                while [ "$#" -gt 0 ]; do
                   mountedlunId[$index]="${mountedlunId[$index]} $1"
                   shift
                done
            fi
            echo "Mounted Lun ID - ${mountedlunId[$index]}"
            lunid=
        fi
    elif [ "X$name" = "Xmvolroot" ]; then
        mountedmntpt[$index]=$value
        echo "Mounted Mount point - ${mountedmntpt[$index]}"
    elif [ "X$name" = "Xmvgname" ]; then
        mountedvgname[$index]=$value
        echo "Mounted Volume group name - ${mountedvgname[$index]}"
    fi

done

OS=`uname -s`
version=`uname -r`
iter=1
while [ $iter -le $index ]; do
    volpath=${mountedvolumeid[$iter]}
    mntpt=${mountedmntpt[$iter]}

    if [ "X$OS" = "XLinux" ]; then
        devicenames=
        for scsi_disk_id in ${mountedlunId[$iter]} ; do
            diskname=
            devicepath=`ls -l /dev/disk/by-id/ 2>/dev/null | grep scsi- | grep -i $scsi_disk_id | awk '{print $NF}'`
            if [ "Xdevicepath" != "X" ]; then
                devicename=`basename $devicepath`
                diskname="/dev/"$devicename
                echo "Found disk $diskname with Lunid $scsi_disk_id"
            fi
            if [ "X$diskname" = "X" ]; then
                echo "Failed to find disk with Lunid $scsi_disk_id"
            else
                devicenames="$devicenames $diskname"
            fi
        done

        devices=`echo $devicenames`
        if [ "X$devices" = "X" ]; then
            echo "Failed to find devices corresponding to mapped Lun ID(s)"
            iter=`expr $iter + 1`
            continue
        fi

        if [ "X${mountedvgname[$iter]}" != "X" ]; then
            vgpath=`dirname $volpath`
            vgname=`basename $vgpath`
            importvg="vgimport "${vgname}
            eval $importvg
            vgactivate="vgchange -a y "${vgname}
            eval $vgactivate
            lv=`lvdisplay $volpath 2>/dev/null | grep "LV Status" | awk '{print $3}'`
            if [ "X$lv" != "Xavailable" ]; then
                echo "Logical Volume $volpath is not available"
            else
                devices=$volpath
            fi
        fi

        echo "Mounting disk $diskname at mountpoint $mntpt"
        mount $devices $mntpt
        echo $mntpt >> $mntptfile

    elif [ "X$OS" = "XHP-UX" ]; then
        devicenames=
        for scsi_disk_id in ${mountedlunId[$iter]} ; do
            diskname=
            if [ "X$version" = "XB.11.31" ]; then
                device_path=`scsimgr -p get_attr all_lun -a device_file -a wwid 2>/dev/null | grep -v "/dev/pt" | grep -i $scsi_disk_id | cut -d ':' -f 1`
                if [ "X$device_path" != "X" ]; then
                    devicename=`basename $device_path`
                    diskname="/dev/disk/"$devicename 
                    echo "Found disk $diskname with Lunid $scsi_disk_id"
                fi
            else
                disks=`ioscan -knfC disk 2>/dev/null | awk '{for(i=1;i<=NF;i++) {print $i}}'|grep '/dev/rdsk'`
                for disk in $disks ; do
                    scsi_lunid=`/opt/act/bin/udsagent fetchlunid $disk 2>&1 | grep Lunid | awk '{print $NF}'`
                    if [ "X$scsi_lunid" = "X$scsi_disk_id" ]; then
                        devicename=`basename $disk`
                        diskname="/dev/dsk/"$devicename
                        echo "Found disk $diskname with Lunid $scsi_disk_id"
                        break
                    fi
                done
            fi
            if [ "X$diskname" = "X" ]; then
                echo "Failed to find disk with Lunid $scsi_disk_id"
            else
                devicenames="$devicenames $diskname"
            fi
        done

        devices=`echo $devicenames`
        if [ "X$devices" = "X" ]; then
            echo "Failed to find devices corresponding to mapped Lun ID(s)"
            iter=`expr $iter + 1`
            continue
        fi
        #
        # get the mount command from Log file and replace the device name if it got changed
        #
        mnt_cmd=`grep "mount -F " $LogFile 2>/dev/null | grep -w "$mntpt" | awk -F ':' '{print $NF}' | tail -1`
        mntcmd=
        if [ "X${mountedvgname[$iter]}" = "X" ]; then
            mntcmd=`echo $mnt_cmd | awk '{$6=a}1' a=$devices`
        else
            vgpath=`dirname $volpath`
            vgname=`basename $vgpath`
            importvg="vgimport  -m /act/"
            importvg="${importvg}${vgname} ${vgname} ${devices}"
            eval $importvg
            vgactivate="vgchange -a y "$vgname
            eval $vgactivate
            lv=`lvdisplay $volpath 2>/dev/null`
            if [ "X$lv" != "X" ]; then
                mntcmd=$mnt_cmd
            else
                echo "Failed to find LV $volpath"
            fi
        fi

        if [ "X$mntcmd" != "X" ]; then
            echo "Running command $mntcmd"
            eval $mntcmd
        fi

    elif [ "X$OS" = "XAIX" ]; then
        devicenames=
        for scsi_disk_id in ${mountedlunId[$iter]} ; do
            diskname=
            disks=`lspv 2>/dev/null | awk '{print $1}'`
            for disk in $disks ; do
                devname="/dev/"$disk
                scsi_lunid=`/opt/act/bin/udsagent fetchlunid $devname 2>&1 | grep Lunid | awk '{print $NF}'`
                if [ "X$scsi_lunid" = "X$scsi_disk_id" ]; then
                    diskname=$devname
                    echo "Found disk $diskname with Lunid $scsi_id"
                    break
                fi
            done
            if [ "X$diskname" = "X" ]; then
                echo "Failed to find disk with Lunid $scsi_disk_id"
            else
                devicenames="$devicenames $diskname"
            fi
        done

        devices=`echo $devicenames`
        if [ "X$devices" = "X" ]; then
            echo "Failed to find devices corresponding to mapped Lun ID(s)"
            iter=`expr $iter + 1`
            continue
        fi
        #
        # get the mount command from Log file and replace the device name if it got changed
        #
        mnt_cmd=`grep "mount -v " $LogFile 2>/dev/null | grep -w "$mntpt" | awk -F ':' '{print $NF}' | tail -1`
        mntcmd=
        if [ "X${mountedvgname[$iter]}" = "X" ]; then
            if [ -z $mnt_cmd ]; then
                mntcmd=`grep "mmmount " $LogFile 2>/dev/null | grep -w "$mntpt" | awk -F ':' '{print $NF}' | tail -1`
            else
                mntcmd=`echo $mnt_cmd | awk '{$6=a}1' a=$devices`
            fi
        else
            pvname=`echo $devicenames | awk '{print $1}'`
            if [ "X$pvname" != "X" ]; then
                vgid=`lqueryvg -v -p $pvname 2>/dev/null`
                if [ "X$vgid" != "X" ]; then
                    vgname=`odmget -q 'attribute=vgserial_id and value=${vgid}' CuAt 2>/dev/null | grep name | awk '{print $3}' | awk -F'"' '{print $2}'`
                    if [ "X$vgname" != "X" ]; then
                        importvg="importvg -y"
                        importvg="${importvg} ${vgname} ${pvname}"
                        eval $importvg
                        vgactivate="varyonvg "$vgname
                        eval $vgactivate
                    fi
                fi
            fi
            lv=`lslv $volpath 2>/dev/null`
            if [ "X$lv" != "X" ]; then
                mntcmd=$mnt_cmd
            else
                echo "Failed to find LV $volpath"
            fi
        fi

        if [ "X$mntcmd" != "X" ]; then
            echo "Running command $mntcmd"
            eval $mntcmd
        fi

    else
       echo "Mounting vol $volpath at mountpoint $mntpt"
       mount $volpath $mntpt
       echo $mntpt >> $mntptfile
    fi

    iter=`expr $iter + 1`
done

rm -f $LogFile 2>/dev/null
exit 0

