#!/bin/sh

#export ACT_JOBTYPE="mount"
#export ACT_PHASE="post"

set -x
cd /act/scripts
conffile=/act/scripts/act_Conf_Target_Primary.conf
source $conffile
echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ $ACT_JOBTYPE = "unmount" ] || [ $ACT_MULTI_OPNAME = "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ $ACT_JOBTYPE == "mount" ] || [ $ACT_MULTI_OPNAME == "mount" ] ; then
   if [ $ACT_PHASE == "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ $ACT_PHASE == "post" ] ; then
        echo "************************* start post during mount ************************"

mntpt=$TARGET_MNT_PNT
chown mysql:mysql /$mntpt
myconf=$mntpt$SRC_DATA_DIR/$mntpt.conf
dumploc=$mntpt$SRC_DATA_DIR


# Start MySQL service 
mysqlsocket=$mntpt$SRC_DATA_DIR$mntpt.sock
echo -e "[client]" > $myconf
echo -e "socket=$mntpt$SRC_DATA_DIR$mntpt.sock" >> $myconf
echo -e "port=$PORT" >> $myconf

echo -e "[mysqld]" >> $myconf
echo -e "datadir=$mntpt$SRC_DATA_DIR" >> $myconf 
echo -e "socket=$mntpt$SRC_DATA_DIR$mntpt.sock" >> $myconf
echo -e "port=$PORT" >> $myconf
echo -e "pid-file=$mntpt$SRC_DATA_DIR$mntpt.pid" >> $myconf
echo -e "log-error=$mntpt$SRC_DATA_DIR/actmysql.log" >> $myconf
echo -e "tmpdir=$mntpt$SRC_TMP_DIR" >> $myconf
echo -e "innodb_data_home_dir=$mntpt$SRC_DATA_DIR" >> $myconf
chown -R mysql:mysql $myconf

mysqlversion=`su - $OSUSER -c "mysql -V" |awk '{print $5}'|awk -F"," '{print $1}'`
echo "MySQL version is $mysqlversion"

if [ "$mysqlversion" == "5.7.20" ]; then
su - $OSUSER -c "mysqld_safe --defaults-file=$myconf --lc-messages-dir=$MESSAGES_DIR &"
retval=$?
else
su - $OSUSER -c "mysqld --defaults-file=$myconf --lc-messages-dir=$MESSAGES_DIR &"
retval=$?
fi
sleep 10;

# Rename database if NEWDB_NAME is set

if [ "$retval" -gt "0" ]; then
echo "Start of MySQL service Failed..!!"
exit 1;
fi

if [ ! -z "$NEWDB_NAME" ]; then
db_exists=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
show databases like '$NEWDB_NAME';
EOF`
  if [ ! -z "$db_exists" ]; then
   echo "RENAME ERROR: DB already exists..!"
  fi
  dbcharset=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -e 'show create database $SRCDB_NAME' -sss "`
  dbcharset=`echo $dbcharset | awk -F'CHARACTER SET ' '{print $2}' | awk '{print $1}'`
  echo $dbcharset

# Create database

su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -e 'create database $NEWDB_NAME CHARACTER SET $dbcharset;'"

dbtables=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select GROUP_CONCAT(TABLE_NAME SEPARATOR ' ') from information_schema.tables where table_schema='$SRCDB_NAME' and TABLE_TYPE='BASE TABLE';
EOF`

# Dump Triggers
dbtriggers=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select GROUP_CONCAT(TRIGGER_NAME SEPARATOR ' ') from information_schema.triggers where trigger_schema='$SRCDB_NAME';
EOF`

MYSQL_DUMP=`echo $MYSQL_CMD | sed '0,/mysql/s//mysqldump/'`
su - $OSUSER -c "$MYSQL_DUMP -S $mysqlsocket $SRCDB_NAME  -d -t -R -E > $dumploc/'$SRCDB_NAME'_triggers.dump"

#Dump Views
dbviews=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select GROUP_CONCAT(TABLE_NAME SEPARATOR ' ') from information_schema.tables where table_schema='$SRCDB_NAME' and TABLE_TYPE='VIEW';
EOF`


if [ "$dbviews" != "NULL" ]; then
su - $OSUSER -c "$MYSQL_DUMP -S $mysqlsocket $SRCDB_NAME $dbviews > $dumploc/'$SRCDB_NAME'_view.dump"
fi

#Drop Triggers before renaming tables
if [ "$dbtriggers" != "NULL" ]; then
set -- $dbtriggers
while [ $# -gt 0 ]
do
        su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -e 'use $SRCDB_NAME;drop trigger $1'"
        shift
done
fi

# Rename tables

for x in $dbtables; do
 su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -e 'SET FOREIGN_KEY_CHECKS=0;RENAME TABLE $SRCDB_NAME.$x to $NEWDB_NAME.$x;'"
done

if [ "$dbviews" != "NULL" ]; then
   echo "Loading Views....!!!"
   su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket $NEWDB_NAME < $dumploc/'$SRCDB_NAME'_view.dump"
fi

if [ "$dbtriggers" != "NULL" ]; then
 echo " Loading Triggers...!!"
 su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket $NEWDB_NAME < $dumploc/'$SRCDB_NAME'_triggers.dump"
fi

# drop old database

dbtables=''
dbtables=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select GROUP_CONCAT(TABLE_NAME SEPARATOR ' ') from information_schema.tables where table_schema='$SRCDB_NAME' and TABLE_TYPE='BASE TABLE';
EOF`

if [ "$dbtables" = "NULL" ]; then
su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -e 'drop database $SRCDB_NAME;'";
fi

#set Col privs
colprivs=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select count(*)  from mysql.columns_priv  where db='$SRCDB_NAME';
EOF`

if [ $colprivs -gt 0 ]; then
su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket" << EOF
update mysql.columns_priv set db='$NEWDB_NAME' WHERE db='$SRCDB_NAME';
EOF
fi
procprivs=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select count(*)  from mysql.procs_priv  where db='$SRCDB_NAME';
EOF`

if [ $procprivs -gt 0 ]; then
su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket" << EOF
update mysql.procs_priv set db='$NEWDB_NAME' WHERE db='$SRCDB_NAME';
EOF
fi

tblprivs=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select count(*)  from mysql.tables_priv  where db='$SRCDB_NAME';
EOF`

if [ $tblprivs -gt 0 ]; then
su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket" << EOF
update mysql.tables_priv set db='$NEWDB_NAME' WHERE db='$SRCDB_NAME';
EOF
fi

dbprivs=`su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -s" << EOF
select count(*)  from mysql.db  where db='$SRCDB_NAME';
EOF`

if [ $dbprivs -gt 0 ]; then
su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket" << EOF
update mysql.db set db='$NEWDB_NAME' WHERE db='$SRCDB_NAME';
EOF
fi

su - $OSUSER -c "$MYSQL_CMD -S $mysqlsocket -e 'flush privileges;'"
retval=$?

if [ "$retval" -gt "0" ]; then
  echo "Rename of the database failed...!!"
  exit 1;
fi
#### Rename DB routine Ends############
fi

if [ -e $dumploc/'$SRCDB_NAME'_triggers.dump ]; then
 rm -f $dumploc/'$SRCDB_NAME'_triggers.dump
fi

if [ -e $dumploc/'$SRCDB_NAME'_view.dump ]; then
 rm -f $dumploc/'$SRCDB_NAME'_view.dump
fi

fi
fi

