#!/bin/sh
#
# OracleAppMount_tstamp.sh: shell script to automate cloning and bring up an Oracle database using backup image copies
#
#usage: OracleAppMount_tstamp.sh <New database sid name> <Oracle Home path> <Backup location mount path> <log mount path> <timestamp yyyymmddhh24mi>"

set +x
nsid=$1
oracleHome=$2
backupLocation=$3
archmnt=$4
tstamp=$5

dataDir=$backupLocation

BASEDIR=$(dirname $0)
#echo $BASEDIR
cd $BASEDIR

source /act/act_scripts/oracleclone/OracleAppMountComFunc.sh

if [ "$#" -ne 5 ]; then
echo "This script needs 5 input parameter"
echo "usage: OracleAppMount_tstamp.sh <New database sid name> <Oracle Home path> <Backup location mount path> <log mount path> <timestamp yyyymmddhh24mi>"
echo " For example: OracleAppMount_tstamp.sh clonedb /home/oracle/app/oracle/product/11.2.0/dbhome_1 /act/mnt/Job_0060901_mountpoint_1341234106825 /act/mnt/vdbxlog 201404021435"
exit 1
fi

#logger for error messages
errMsgFile=$nsid"_errMsg"

function apmlogger {
     errmsg="$1"
     if [ ! -z "$errmsg" ]; then
        echo "$errmsg" > $errMsgFile
     fi
}

function apmappend {
     errmsg="$1"
     echo " $errmsg" >> $errMsgFile
}

function geterrmsg {
     ecode=$1
     if [ -z "$ecode" ]; then
        echo "missing error code when retrieving error message"
     fi
     msgrepo=/act/act_scripts/apmerrmsg.sh
     msg=`grep $ecode"=" $msgrepo | awk -F'=' '{print $NF}'`
     if [ ! -z "$msg" ]; then
        echo "$msg"
     else
        echo "failed to retrieve error message for $ecode"
     fi
}

function logerrormsg {
     ecode=$1
     appender=$2

     emsg=`geterrmsg $ecode`
     apmlogger "$emsg"
     if [ ! -z "$$appender" ]; then
        apmappend $appender
     fi
     echo `$tdate`$nsid": $emsg $appender"
}
#logger

if [ ! -z "$isrestore" ] && [ "$isrestore" = "true" ]; then
ORACLE_HOME=$oracleHome
export ORACLE_HOME
PATH=$ORAHOME/bin:$PATH
export PATH
ORACLE_SID=$nsid
export ORACLE_SID
TNS_ADMIN=$tnsadmindir
export TNS_ADMIN
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

if [ ! -z "$DB_TZ" ]; then
echo " ********** setting TZ:$DB_TZ  ***************"
TZ=$DB_TZ
export TZ
fi

$ORACLE_HOME/bin/sqlplus -s / as sysdba << EOF
set heading off
set feedback off
startup mount;
alter database end backup;
exit;
EOF

echo " ********** convert the tstamp to db timezone DB_TZ:$DB_TZ if different  ***************"
if [ ! -z "$DB_TZ" ]; then
tstamp=`$ORACLE_HOME/bin/sqlplus -S / as sysdba @/act/act_scripts/TZconvertApp.sql $tstamp $DB_TZ`
retval=$?
if [ $retval -gt 0 ]; then
echo "********** error converting the db timezone DB_TZ:$DB_TZ  ***************"
exit 110;
fi
fi

catalogstr=`generate_catalog_str $archmnt`

export NLS_DATE_FORMAT="yyyymmddhh24mi"
rmancmd="run
        {
        allocate channel d1 device type disk;
        allocate channel d2 device type disk;
        $catalogstr
        recover database until time \"to_date('$tstamp','yyyymmddhh24mi')\";
        release channel d1;
        release channel d2;
        }"

dbrecover="/act/act_scripts/oracleclone/dbrecover_"$nsid".txt"
if [ ! -f "$dbrecover" ]; then
   touch $dbrecover
#   chmod 777 $dbrecover
fi
ORACLE_HOME=$oracleHome
export ORACLE_HOME
PATH=$ORAHOME/bin:$PATH
export PATH
ORACLE_SID=$nsid
export ORACLE_SID
TNS_ADMIN=$tnsadmindir
export TNS_ADMIN
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

retv1=`$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
                 $rmancmd
                 exit
EOF`

if [ $? -ne 0 ]; then
   logerrormsg EM_DB_RECOVERY
   exit 1
fi

$ORACLE_HOME/bin/sqlplus -s / as sysdba << EOF
set heading off
set feedback off
alter database open resetlogs;
exit;
EOF
if [ $? -ne 0 ]; then
   logerrormsg EM_DB_OPEN
   exit 1
fi

exit 0
#inband restore
fi


BASEDIR=$(dirname $0)
echo $BASEDIR
cd $BASEDIR
# correctly set the ORACLE_BASE AND ORACLE_HOME  for your environment
export ORACLE_HOME=$oracleHome
export ORACLE_BASE=$dataDir
export PATH=$ORACLE_HOME/bin:$PATH
LD_LIBRARY_PATH=$ORACLE_HOME/lib
export LD_LIBRARY_PATH

if [ ! -d $ORACLE_HOME ]; then
   logerrormsg EM_ORACLE_HOME
   exit 1
fi

if [ ! -d $ORACLE_BASE ]; then
   logerrormsg EM_FIND_BACKUP_LOCATION
   exit 1
fi

if [ ! -d $backupLocation ]; then
   logerrormsg EM_FIND_BACKUP_LOCATION
   exit 1
fi

sqlplus=$ORACLE_HOME/bin/sqlplus

psid=`get_teardown_sid $dataDir`
remountflag=$isremount

echo "******************* remount flag: $remountflag ********************"
echo "******************* psid: $psid ********************"
if [ $remountflag = "true" ]; then
    echo "Remount $psid"
    /act/act_scripts/oracleclone/OracleAppRemount.sh $nsid $ORACLE_HOME $dataDir
    if [ $? != 0 ]; then
       logerrormsg EM_REMOUNT "$psid to $nsid"
       exit 1 
    fi
    exit 0
else

# get the last archivelog name with path
mname=$backupLocation
echo $mname

# get the production sid name from mount
if [ ! -z "$ischild" ] && [ "$ischild" = "true" ]; then
   #check prev appAware mount
   pmpdir=$dataDir"/pmpkeep"
   if [ -e "$pmpdir" ]; then
      # if prevsid is set, use it, this is the last appAware mount SID, if not the last one should the latest SID
      if [ -z "$prevsid" ]; then
         getpname=`ls -ltr $dataDir/init*.ora | tail -1 | awk '{print $NF}'`
      else
         getpname=`ls -ltr $dataDir/init$prevsid.ora | awk '{print $NF}'`
      fi
      pname=$(basename $getpname)
      echo $pname
      pname=`echo $pname | cut -d"." -f1`
      osid=`echo $pname | cut -c5-`
   else
      getpname=`ls -ltr $mname | grep '__backup.ora'`
      echo $getpname
      pname=`echo $getpname | awk '{print $NF}'`
      echo $pname
      osid=`echo $pname | awk -F"__" '{print $(NF-1)}'`
   fi
else
   getpname=`ls -ltr $mname | grep '__backup.ora'`
   echo $getpname
   pname=`echo $getpname | awk '{print $NF}'`
   echo $pname
   osid=`echo $pname | awk -F"__" '{print $(NF-1)}'`
fi

echo $getpname
#pname=`echo $getpname | awk '{print $NF}'`
#echo $pname
#osid=`echo $pname | awk -F"__" '{print $(NF-1)}'`
echo "***************"
echo $osid
osid1=$osid"1"
osid2=$osid"2"
osid3=$osid"3"
osid4=$osid"4"
osid5=$osid"5"
osid6=$osid"6"
echo $osid1
echo $osid2

if [ ! -z "$ischild" ] && [ "$ischild" = "true" ]; then
   prevorafile=$mname/"init"$osid".ora"
   if [ -e $prevorafile ]; then
      cp $prevorafile $mname/$osid"__backup.ora"
   fi
fi

orafile=$mname/$osid"__backup.ora"
cp $orafile $orafile".orig"
echo $orafile
backupdir=$mname/datafile
echo $backupdir

dbfsPath=$mname/$osid"_datafiles_readonly_catalog.conf"
echo "read only tablespace file path : $dbfsPath"

# keep mountpoint with the mount for remount
echo "$backupLocation" > $backupLocation/pmpkeep

openDatabase="openDatabase_"$nsid".sql"
recoverDatabase="recoverDatabase_"$nsid".sql"
verifyDatabase="verifyDatabase.sql"
CLONE_CRT_SQL="Clone_createCtrlFile_"$nsid"_g.sql"
openDatabase1="openDatabase1_"$nsid".sql"
openDBlog="openDBlog_"$nsid".txt"

openPDatabase="openPDatabase_"$nsid".sql"
openTmpDatabase="openTmpDatabase_"$nsid".sql"
openTmpDatabase1="openTmpDatabase1_"$nsid".sql"
addUnnamed="addUnnamed_"$nsid".sql"
tabfilelist="tabfilelist_"$nsid
undofilelist="undofilelist_"$nsid".sql"
offlinefilelist="offlinefilelist_"$nsid
pluggablelist="pluggablelist_"$nsid
pglist="pglist_"$nsid
createRedolog="createRedolog_"$nsid".sql"

runnidsql="runnidrotbs"_$nsid".sql"

#sqlplus=$ORACLE_HOME/bin/sqlplus

if [ ! -d $mname ]; then
   logerrormsg EM_FIND_BACKUP_LOCATION
   exit 1
fi

if [ -z $nsid ]; then
   logerrormsg EM_NEW_SID
   exit 1
fi

if [ -z $orafile ]; then
   logerrormsg EM_INIT_FILE
   exit 1
fi

if [ ! -d $backupdir ]; then
   logerrormsg EM_FIND_BACKUP_LOCATION
   exit 1
fi


oobase=$ORACLE_BASE
removesid=rsid.sh
# replace RAC nodes parameter except node 1
echo "sed -i '/$osid2/d' $orafile" > $removesid
echo "sed -i '/$osid3/d' $orafile" >> $removesid
echo "sed -i '/$osid4/d' $orafile" >> $removesid
echo "sed -i '/$osid5/d' $orafile" >> $removesid
echo "sed -i '/$osid6/d' $orafile" >> $removesid
sed -i '/__/d' $orafile 

chmod 755 $removesid

./rsid.sh
# replace node 1 SID with new SID
sed -i "s/'$osid1'/'$nsid'/g" $orafile

if [ -e $removesid ]; then
   rm -f $removesid
fi
# make required directory structures
mkdir -p $ORACLE_BASE/oradata/$nsid
mkdir -p $ORACLE_BASE/flash_recovery_area/$nsid
mkdir -p $ORACLE_BASE/admin/$nsid/adump
mkdir -p $ORACLE_BASE/admin/$nsid/dpdump
mkdir -p $ORACLE_BASE/admin/$nsid/control
mkdir -p $ORACLE_BASE/admin/$nsid/archivelog

controlfile="*.control_files='$ORACLE_BASE/admin/$nsid/control/control01.ctl','$ORACLE_BASE/admin/$nsid/control/control02.ctl'"
dbcreatefile="*.db_create_file_dest='$ORACLE_BASE/oradata/$nsid'"
dbrecoveryfile="*.db_recovery_file_dest='$ORACLE_BASE/flash_recovery_area/$nsid'"
if [ -z "$diagnostic_dest" ]; then
  mkdir -p $ORACLE_HOME/../../admin/$nsid/adump
  mkdir -p $ORACLE_HOME/../../diag
else
  mkdir -p $diagnostic_dest/$nsid
  mkdir -p $diagnostic_dest/$nsid/admin/adump
fi
logarchivedest="*.log_archive_dest_1='LOCATION=$ORACLE_BASE/admin/$nsid/archivelog/'"

nsidlen=`echo -n $nsid | wc -m`
if [ $nsidlen -gt 8 ]; then
newdb_name=`echo $nsid | cut -c1-8`
db_unique_name=$nsid
else
newdb_name=$nsid
fi

dbname="*.db_name='$newdb_name'"
dbuniquename="*.db_unique_name='$nsid'"

jqprocess="*.job_queue_processes=0"

if [ -z "$totalmemory" ]; then
    #sgatarget="*.sga_target=800m"
    #pgatarget="*.pga_aggregate_target=400m"
    echo " memory size is not defined "
    echo "source memory setup will be used"
else
    if [ ! -z "$totalmemory" ] && [ -z "$sgapct" ]; then
        echo " SGA is not defined "
        echo "memory_target setup will be used"
        sed -i '/sga_max_size=/d' $orafile
        sed -i '/sga_target=/d' $orafile
        sed -i '/pga_aggregate_target=/d' $orafile
        sed -i '/memory_target=/d' $orafile
        sed -i '/memory_max_target=/d' $orafile
        sed -i '/streams_pool_size=/d' $orafile
        sed -i '/db_cache_size/d' $orafile
        sed -i '/shared_pool_size=/d' $orafile
        sed -i '/db_keep_cache_size/d' $orafile
        sed -i '/shared_pool_reserved_size=/d' $orafile
        sed -i '/large_pool_size/d' $orafile
        totalmemoryM=$totalmemory"m"
        mtarget="*.memory_target=$totalmemoryM"
        echo "$mtarget" >> $orafile
    else
        echo " total memory and SGA is defined "
        echo "sga and pga setup will be used"
        sed -i '/sga_max_size=/d' $orafile
        sed -i '/sga_target=/d' $orafile
        sed -i '/pga_aggregate_target=/d' $orafile
        sed -i '/memory_target=/d' $orafile
        sed -i '/memory_max_target=/d' $orafile
        sed -i '/streams_pool_size=/d' $orafile
        sed -i '/db_cache_size/d' $orafile
        sed -i '/shared_pool_size=/d' $orafile
        sed -i '/db_keep_cache_size/d' $orafile
        sed -i '/shared_pool_reserved_size=/d' $orafile
        sed -i '/large_pool_size/d' $orafile
        sgc=$(($totalmemory*$sgapct/100))
        sgam=$sgc"m"
        sgatarget="*.sga_target=$sgam"
        pgc=$(($totalmemory - $sgc ))
        pgam=$pgc"m"
        pgatarget="*.pga_aggregate_target=$pgam"
        echo "$sgatarget" >> $orafile
        echo "$pgatarget" >> $orafile

        lps=$((($sgc*10)/100))
        if [ "$lps" -gt "200" ]; then
           lapsm=$lps"m"
        else
           lapsm=200m
        fi

        largepool="*.large_pool_size=$lapsm"
        echo -e "$largepool" >> $orafile

        if [ ! -z "$shared_pool_size" ]; then
            sps=$shared_pool_size"m"
            sharedpoolsize="*.shared_pool_size=$sps"
            echo -e "$sharedpoolsize" >> $orafile
        fi

        if [ ! -z "$db_cache_size" ]; then
           dcs=$db_cache_size"m"
           dbcachesize="*.db_cache_size=$dcs"
           echo -e "$dbcachesize" >> $orafile
        fi
    fi
 fi

sed -i '/db_recovery_file_dest_size=/d' $orafile
if [ ! -z "$db_recovery_file_dest_size" ]; then
drfds=$db_recovery_file_dest_size"m"
dbrecoveryfilesize="*.db_recovery_file_dest_size=$drfds"
else
dbrecoveryfilesize="*.db_recovery_file_dest_size=50g"
fi

sed -i '/processes=/d' $orafile
if [ ! -z "$processes" ]; then
process="*.processes=$processes"
else
process="*.processes=500"
fi

if [ ! -z "$open_cursors" ]; then
sed -i '/open_cursors=/d' $orafile
opencursors="*.open_cursors=$open_cursors"
echo -e "$opencursors" >> $orafile
fi

sed -i '/oracle_base=/d' $orafile
sed -i '/control_files=/d' $orafile
sed -i '/audit_file_dest=/d' $orafile
sed -i '/db_create_file_dest=/d' $orafile
sed -i '/db_recovery_file_dest=/d' $orafile
sed -i '/diagnostic_dest=/d' $orafile
sed -i '/log_archive_dest/d' $orafile
sed -i '/cluster_database=/d' $orafile
sed -i '/remote_listener/d' $orafile
sed -i '/local_listener/d' $orafile
sed -i '/user_dump_dest/d' $orafile
sed -i '/core_dump_dest/d' $orafile
sed -i '/db_name/d' $orafile
sed -i '/dispatchers=/d' $orafile
sed -i '/instance_number=/d' $orafile
sed -i '/thread=/d' $orafile
sed -i '/db_unique_name=/d' $orafile
sed -i '/$osid2/d' $orafile
sed -i '/$osid/d' $orafile

sed -i '/cluster_interconnects/d' $orafile
sed -i '/_diag_adr_trace_dest/d' $orafile

sed -i '/cluster_database_instances/d' $orafile
sed -i '/instance_name/d' $orafile
sed -i '/standby_archive_dest/d' $orafile
sed -i '/background_dump_dest/d' $orafile

sed -i '/db_create_online_log_dest/d' $orafile

sed -i '/_fix_control/d' $orafile
sed -i '/event/d' $orafile

sed -i '/use_large_pages/d' $orafile

echo -e "$controlfile" >> $orafile
echo -e "$dbname" >> $orafile
echo -e "$dbuniquename" >> $orafile
echo -e "$dbcreatefile" >> $orafile
echo -e "$dbrecoveryfile" >> $orafile
echo -e "$dbrecoveryfilesize" >> $orafile
#echo -e "$dbdiagnostic" >> $orafile
if [ ! -z "$diagnostic_dest" ]; then
 dbdiagnostic="*.diagnostic_dest='$diagnostic_dest'"
 echo -e "$dbdiagnostic" >> $orafile
 adest="*.audit_file_dest='$diagnostic_dest/$nsid/admin/adump'"
 echo -e "$adest" >> $orafile
else
 auditfile="*.audit_file_dest='$ORACLE_BASE/admin/$nsid/adump'"
 echo -e "$auditfile" >> $orafile
fi
echo -e "$logarchivedest" >> $orafile
echo -e "$process" >> $orafile
echo -e "$jqprocess" >> $orafile

sed -i '/disk_asynch_io/d' $orafile
diskasyncio="*.disk_asynch_io='TRUE'"
echo -e "$diskasyncio" >> $orafile

# generate a controlfile creation script
./Clone_generateCtrlFileScript.sh $nsid $backupdir $ORACLE_BASE"/oradata"


if [ $? -ne 0 ]; then
   logerrormsg EM_GENERATE_CONTROLFILE
   exit 1
fi

#create sql file to open the database and add tempfile
if [ -e $openDatabase ]; then
   #echo "removing $openDatabase before generating a new one"
   rm -f $openDatabase
fi

if [ -e $openDatabase1 ]; then
   #echo "removing $openDatabase1 before generating a new one"
   rm -f $openDatabase1
fi

if [ -e $recoverDatabase ]; then
   #echo "removing $recoverDatabase before generating a new one"
   rm -f $recoverDatabase
fi

if [ -e $addUnnamed ]; then
   #echo "removing $openDatabase before generating a new one"
   rm -f $addUnnamed
fi

if [ -e $tabfilelist ]; then
   #echo "removing $tabfilelist before generating a new one"
   rm -f $tabfilelist
fi

if [ -e $undofilelist ]; then
  # echo "removing $undofilelist "
   rm -f $undofilelist
fi

if [ -e $openDBlog ]; then
  # echo "removing $openDBlog "
   rm -f $openDBlog
fi

# now export new SID, do this outside of shell, before invoke this shell
export ORACLE_SID=$nsid

# copy ora file
#cp $orafile $ORACLE_HOME/dbs/init"$nsid".ora
initf=$ORACLE_HOME/dbs/init"$nsid".ora
spf=$ORACLE_HOME/dbs/spfile"$nsid".ora

# keep a copy of ora file under ORACLE_BASE
cp $orafile $ORACLE_BASE/init"$nsid".ora
# start the database for cloning
$sqlplus / as sysdba @Clone.sql $orafile $spf

if [ $? -ne 0 ]; then
   logerrormsg EM_GENERATE_CONTROLFILE
   exit 1
fi

echo -e 'SPFILE=$spf' > $initf

$sqlplus / as sysdba @mountDatabase.sql $CLONE_CRT_SQL

if [ $? -ne 0 ]; then
   logerrormsg EM_CREATE_CONTROLFILE
   exit 1
fi


echo " ********** convert the tstamp to db timezone DB_TZ:$DB_TZ if different  ***************"
if [ ! -z "$DB_TZ" ]; then
tstamp=`$sqlplus -S / as sysdba @/act/act_scripts/TZconvertApp.sql $tstamp $DB_TZ`
retval=$?
if [ $retval -gt 0 ]; then
echo "********** error converting the db timezone DB_TZ:$DB_TZ  ***************"
exit 110;
fi
fi

if [ -e $CLONE_CRT_SQL ]; then
echo "control file remove"
   rm -f $CLONE_CRT_SQL
fi

dbrecover="dbrecover_"$nsid".txt"
export ORACLE_SID=$nsid

vnum=0
vnum=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
set feedback off
select count(*) from v\$instance where version like '12%';
exit;
EOF1`
echo "********************* vnum=$vnum ***********"
pdbcnt=0
if [ "$vnum" -gt "0" ]; then
  pdbcnt=`$sqlplus -S / as sysdba << 'EOF1'
  set head off;
  set feedback off
  select count(*) from v\$pdbs where name like 'PDB%SEED';
  exit;
EOF1`
fi
export vnum
export pdbcnt

echo "******* recovering database *********** "
echo " recovering database: pdbcnt=$pdbcnt ischild=$ischild " > $dbrecover

#if [ $pdbcnt -gt 0 ]; then
$sqlplus / as sysdba << EOF
alter database archivelog;
exit;
EOF
#fi

./recoverRman_tstamp.sh $mname $nsid $archmnt $tstamp $dbfsPath

retval=0
recoverdb=true
if [ ! -z "$rrecovery" ] && [ "$rrecovery" = "false" ]; then
   recoverdb=false
fi

if [ $pdbcnt -eq 0 ]; then
$sqlplus / as sysdba @newDatafile.sql
nstmtsql=nstmtsql.sql
echo "set head off;" > $nstmtsql
echo "set feedback off" >> $nstmtsql
echo "select to_char(to_date('$tstamp'+1,'yyyymmddhh24mi'),'yyyymmddhh24mi') from dual;" >> $nstmtsql
echo "exit;" >> $nstmtsql
ntstamp=`$sqlplus -S / as sysdba @$nstmtsql`
rm -f $nstmtsql
./recoverRman_tstamp.sh $mname $nsid $archmnt $ntstamp $dbfsPath
fi

if [ "$pdbcnt" -gt "0" ]; then
$sqlplus -S / as sysdba << EOF > $offlinefilelist
set head off;
set feed off;
set echo off;
select file# || ':' || ltrim(substr(name,instr(name,'/',-1)),'/')  from v\$datafile where status='RECOVER' and enabled = 'READ WRITE';
exit;
EOF
chmod 755 $offlinefilelist

while read line
do
    set -- $line
    filenum=$1
        echo "filenumber is: $filenum"
if [ "x$filenum" != "x" ]; then
fnm=`echo $filenum | awk -F':' '{print $1}'`
fname=`echo $filenum | awk -F':' '{print $NF}'`
$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
run {
SET NEWNAME FOR DATAFILE $fnm TO '$mname/datafile/$fname';
alter database create datafile $fnm as new;
}
exit;
EOF
fi
done < $offlinefilelist
fi

if [ $pdbcnt -gt 0 ]; then
$sqlplus / as sysdba @newDatafile.sql
nstmtsql=nstmtsql.sql
echo "set head off;" > $nstmtsql
echo "set feedback off" >> $nstmtsql
echo "select to_char(to_date('$tstamp'+1,'yyyymmddhh24mi'),'yyyymmddhh24mi') from dual;" >> $nstmtsql
echo "exit;" >> $nstmtsql
ntstamp=`$sqlplus -S / as sysdba @$nstmtsql`
rm -f $nstmtsql
./recoverRman_tstamp.sh $mname $nsid $archmnt $ntstamp $dbfsPath
fi

if [ "$recoverdb" = "true" ]; then
echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $openDatabase
echo -e "alter database open resetlogs;" >> $openDatabase
echo -e "CREATE temporary TABLESPACE TEMP_CL TEMPFILE '$ORACLE_BASE/oradata/$nsid/temp_cl01.dbf' size 5g autoextend on;" >> $openDatabase
echo -e "exit" >> $openDatabase

echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $openPDatabase
echo -e "alter database open;" >> $openPDatabase
echo -e "alter database enable block change tracking;" >> $openPDatabase
echo -e "CREATE temporary TABLESPACE TEMP_CL TEMPFILE '$ORACLE_BASE/oradata/$nsid/temp_cl01.dbf' size 5g autoextend on maxsize unlimited;" >> $openPDatabase
echo -e "shutdown immediate;" >> $openPDatabase
echo -e "startup mount;" >> $openPDatabase
echo -e "recover database until cancel;" >> $openPDatabase
echo -e "alter database open resetlogs;" >> $openPDatabase
echo -e "exit" >> $openPDatabase

openDBlog="openDBlog_"$nsid".txt"

if [ -e $openDBlog ]; then
#   echo "removing $openDBlog before generating a new one"
   rm -f $openDBlog
fi

echo -e "Opening Database is started..." > $openDBlog

if [ "$pdbcnt" -gt "0" ]; then
$sqlplus / as sysdba @recoverPDBSeed.sql >> $openDBlog
fi


echo "*********************** opendb **************************"
export ORACLE_SID=$nsid
$sqlplus / as sysdba @$openDatabase >> $openDBlog
retval1=$?
echo "*********************** retval:$retval1 **************************"

orabug=`grep ORA-01092 $openDBlog`
if [ -z "$orabug" ]; then
orabug=`grep ORA-03113 $openDBlog`
fi

if [ ! -z "$orabug" ]; then
sleep 60
$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
run {
startup mount;
recover database;
}
exit;
EOF
fi
if [ "$pdbcnt" -gt "0" ]; then
$sqlplus -S / as sysdba << EOF > $offlinefilelist
set head off;
set feed off;
set echo off;
select file# from v\$datafile where status='OFFLINE';
exit;
EOF
chmod 755 $offlinefilelist

while read line
do
    set -- $line
    filenum=$1
        echo "filenumber is: $filenum"
if [ "x$filenum" != "x" ]; then
$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
run {
recover datafile $filenum;
alter database datafile $filenum online;
}
exit;
EOF
fi
done < $offlinefilelist
fi
if [ ! -z "$orabug" ]; then
$sqlplus / as sysdba @$openPDatabase >> $openDBlog
retval1=$?
fi

if [ "$retval1" -gt "0" ]
then
 while [ "$retval1" -gt "0" ]; do

undfcnt=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
select count(*) from v\$datafile where name like '%/UNNAMED%';
exit;
EOF1`

  if [ "$undfcnt" -gt "0" ]; then

      undfnum=`$sqlplus -S / as sysdba " << 'EOF2'
        set head off;
        select file# from v\$datafile where name like '%/UNNAMED%';
        exit;
EOF2`

        echo -e "WHENEVER SQLERROR EXIT SQL.SQLCODE" > $addUnnamed
        echo -e "alter database create datafile $undfnum as new;" >> $addUnnamed
        echo -e "exit" >> $addUnnamed

        $sqlplus / as sysdba @$addUnnamed
       ./recoverRman_tstamp.sh $mname $nsid $archmnt $tstamp $dbfsPath
        $sqlplus / as sysdba @$openDatabase >> $openDBlog
        retval1=$?;
        orabug=`grep ORA-01092 $openDBlog`
        if [ -z "$orabug" ]; then
                orabug=`grep ORA-03113 $openDBlog`
        fi
        if [ ! -z "$orabug" ]; then
                echo "******************" >> $openDBlog
                echo "ACT_PDB_TF" >> $openDBlog
$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
run {
startup mount;
recover database;
}
exit;
EOF
                $sqlplus / as sysdba @$openPDatabase >> $openDBlog
                retval1=$?
                oraissue=`awk '/ACT_PDB_TF/{y=1;next}y' $openDBlog | grep ORA-01196`
                if [ ! -z "$oraissue" ]; then
                   logerrormsg EM_DB_OPEN "$oraissue"
                   exit 1
                   #     exit $oraissue
                fi
         fi
   else
     retval1=0
   fi
retval1=$?;
 done
 fi

ggusername=`$sqlplus -S / as sysdba << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none;
select distinct owner from dba_segments where segment_name like 'GGS_%' and owner<>'SYS' and owner<>'SYSTEM' and rownum<2;
exit;
EOF1`

if [ ! -z "$ggusername" ]; then
echo "Golden Gate User:$ggusername"
$sqlplus -S / as sysdba <<EOF1
WHENEVER SQLERROR EXIT SQL.SQLCODE
alter user "$ggusername" account lock;
startup force restrict;
drop user "$ggusername" cascade;
exit;
EOF1
retval=$?
echo "Drop User: $retval"

if [ $retval -gt 0 ]; then
$sqlplus -S / as sysdba <<EOF1
WHENEVER SQLERROR EXIT SQL.SQLCODE
shutdown abort;
startup force restrict;
drop user "$ggusername" cascade;
exit;
EOF1
fi
fi

opencnt=`$sqlplus -S / as sysdba << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?
if [ $retval -gt 0 ]; then
$sqlplus / as sysdba << EOF
startup force restrict;
exit;
EOF
fi

if [ "$pdbcnt" -gt "0" ]; then
$sqlplus -S / as sysdba << EOF > $pluggablelist
set head off;
set feed off;
set echo off;
select name from v\$pdbs where con_id in (select con_id from v\$recover_file where error = 'UNKNOWN ERROR');
exit;
EOF
chmod 755 $pluggablelist

while read line
do
    set -- $line
    filenum=$1
        echo "filenumber is: $filenum"
if [ "x$filenum" != "x" ]; then
$ORACLE_HOME/bin/rman target / log=$dbrecover append << EOF
run {
recover pluggable database $filenum;
}
exit;
EOF
fi
done < $pluggablelist

$ORACLE_HOME/bin/rman target / log=$dbrecover append << 'EOF'
run {
alter session set "_oracle_script"=true;
ALTER pluggable database all open;
alter pluggable database all save state;
}
exit;
EOF
fi

opencnt=`$sqlplus -S / as sysdba << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?

if [ $retval -gt 0 ]; then
 oraerror=`grep ORA-01092 $openDBlog`
 logerrormsg EM_DB_READ_WRITE "$oraerror"
 exit 1;
elif [ $opencnt -eq 0 ]; then
 oraerror=`grep ORA-01092 $openDBlog`
 logerrormsg EM_DB_READ_WRITE "$oraerror"
 exit 1;
fi

tempg=0
tempg=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
set feedback off
select count(*) from dba_tablespace_groups;
exit;
EOF1`

if [ "$tempg" -gt "0" ]; then
tempgn=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
set feed off;
set echo off;
select distinct group_name from dba_tablespace_groups where group_name in (select property_value from database_properties where property_name = 'DEFAULT_TEMP_TABLESPACE');
exit;
EOF1`
fi
tabgroup="tabgroup_"$nsid".txt"

if [ "x$tempgn" != "x" ]; then
$sqlplus -S / as sysdba << 'EOF1' > $tabgroup
set head off;
set feed off;
set echo off;
select tablespace_name from dba_tablespace_groups where group_name in (select distinct group_name from dba_tablespace_groups where group_name in (select property_value from database_properties where property_name = 'DEFAULT_TEMP_TABLESPACE'));
exit;
EOF1
chmod 755 $tabgroup
else
tempn=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
select property_value FROM DATABASE_PROPERTIES where PROPERTY_NAME='DEFAULT_TEMP_TABLESPACE';
exit;
EOF1`
fi

if [ "$pdbcnt" -eq "0" ]; then
$sqlplus -S / as sysdba << EOF > $tabfilelist
set head off;
set feed off;
set echo off;
select tb.name tabname from v\$tablespace tb where tb.name != 'TEMP_CL' and ts# not in (select ts# from v\$datafile);
exit;
EOF
else
$sqlplus -S / as sysdba << EOF > $tabfilelist
set head off;
set feed off;
set echo off;
select tb.name tabname from v\$tablespace tb where tb.name != 'TEMP_CL' and con_id not in (select con_id from v\$pdbs where name != 'PDB\%SEED') and ts# not in (select ts# from v\$datafile where con_id not in (select con_id from v\$pdbs where name != 'PDB\%SEED'));
exit;
EOF
fi

echo -e "alter database default temporary tablespace TEMP_CL;" > $openDatabase1

while read line
do
    set -- $line
    tabname=$1
        echo "outside if $tabname"
        if [ "x$tabname" != "x" ]; then
        echo "inside if $tabname"
         tbfname=$tabname"_01.dbf"
        tmpstr=" '$ORACLE_BASE/oradata/$nsid/$tbfname' size 5g autoextend on;"
        echo -e "drop tablespace $tabname including contents and datafiles;" >> $openDatabase1
        echo -e "create temporary tablespace $tabname tempfile $tmpstr" >> $openDatabase1
        fi
done < $tabfilelist

echo -e "shutdown immediate;" >> $openDatabase1
echo -e "startup;" >> $openDatabase1

if [ "x$tempgn" != "x" ]; then
while read line
do
    set -- $line
    tabgname=$1
        echo "tab group if $tabgname"
        if [ "x$tabgname" != "x" ]; then
        echo "in tab group if $tabgname"
        echo -e "alter tablespace $tabgname tablespace group $tempgn;" >> $openDatabase1
        fi
done < $tabgroup

echo -e "alter database default temporary tablespace $tempgn;" >> $openDatabase1
else
echo -e "alter database default temporary tablespace $tempn;" >> $openDatabase1
fi

echo -e "exit;" >> $openDatabase1

if [ "$nonid" != "true" ]; then
if [ ! -f "$dbfsPath" ]; then
   roTBS='NONE'
else
   roTBS=`cat $dbfsPath`
fi

roCTBS=''
for i in $(echo $roTBS | tr "," "\n")
do
    echo "$i"
    if [ -z "$roCTBS" ]; then
        roCTBS="'''"${i}"'''"
    else
        roCTBS=${roCTBS}"\n'''"${i}"'''"
    fi
done

echo "$roCTBS"

ctbs_input=''
echo -e "$roCTBS" > roCTBS_output.txt
ctbs_input=`cat roCTBS_output.txt`
echo "$ctbs_input"

if [ -e "$runnidsql" ]; then
rm -f $runnidsql
fi

while read line
do
    set -- $line
    readtabname=$1
        if [ "x$readtabname" != "x" ]; then
        echo -e "@nidROTBS.sql $readtabname ONLINE" >> $runnidsql
        echo -e "@nidROTBS.sql $readtabname 'READ WRITE'" >> $runnidsql
        fi
done < roCTBS_output.txt
chmod 755 $runnidsql

$sqlplus / as sysdba << EOF
@$runnidsql
shutdown immediate;
startup mount;
exit;
EOF
change_dbid $nsid $ORACLE_HOME
if [ ! -f "$runnidsql" ]; then
 rm -f $runnidsql
fi

while read line
do
    set -- $line
    readtabname=$1
        if [ "x$readtabname" != "x" ]; then
        echo -e "@nidROTBS.sql $readtabname 'READ ONLY'" >> $runnidsql
        fi
done < roCTBS_output.txt
chmod 755 $runnidsql

$sqlplus / as sysdba << EOF
startup mount;
alter database open resetlogs;
@$runnidsql
exit;
EOF
fi

retval=$?
if [ $retval != 0 ]; then
   logerrormsg EM_NID
   exit 1
fi
if [ ! -f "roCTBS_output.txt" ]; then
 rm -f roCTBS_output.txt
fi

export ORACLE_SID=$nsid
$sqlplus / as sysdba @$openDatabase1 >> $openDBlog

if [ -e $dbrecover ]; then
   echo "removing $dbrecover "
#   rm -f $dbrecover
fi

if [ -e $recoverDatabase ]; then
   echo "removing $recoverDatabase "
#   rm -f $recoverDatabase
fi

if [ -e $openDatabase ]; then
   echo "removing $openDatabase "
   rm -f $openDatabase
fi

if [ -e $openDatabase1 ]; then
   echo "removing $openDatabase1 "
   rm -f $openDatabase1
fi

if [ -e $tabfilelist ]; then
   #echo "removing $tabfilelist before generating a new one"
   rm -f $tabfilelist
fi

if [ -e $verifyDatabase ]; then
#   echo "removing $verifyDatabase before generating a new one"
   rm -f $verifyDatabase
fi

if [ -e $openTmpDatabase ]; then
   echo "removing $openTmpDatabase "
   rm -f $openTmpDatabase
fi
if [ -e $addUnnamed ]; then
   echo "removing $addUnnamed "
   rm -f $addUnnamed
fi
if [ -e $verifyDatabase ]; then
   echo "removing $verifyDatabase before generating a new one"
   rm -f $verifyDatabase
fi

if [ -e $openDBlog ]; then
   echo "removing $openDBlog "
   #rm -f $openDBlog
fi

if [ -e $tabfilelist ]; then
   echo "removing $tabfilelist "
   rm -f $tabfilelist
fi

if [ -e $offlinefilelist ]; then
   echo "removing $offlinefilelist "
   #rm -f $offlinefilelist
fi

if [ -e $pluggablelist ]; then
   echo "removing $pluggablelist "
   #rm -f $pluggablelist
fi

if [ -e $createRedolog ]; then
  # echo "removing $createRedolog "
   rm -f $createRedolog
fi

opencnt=`$sqlplus -S / as sysdba << 'EOF1'
WHENEVER SQLERROR EXIT SQL.SQLCODE
set head off;
set feedback off
set newpage none
select count(*) from v\$database where open_mode = 'READ WRITE';
exit;
EOF1`
retval=$?

if [ $opencnt -eq 0 ]; then
   logerrormsg EM_DB_READ_WRITE
   exit 1;
fi

export ORACLE_SID=$nsid
$sqlplus / as sysdba  << EOF3
drop tablespace TEMP_CL including contents and datafiles;
exit;
EOF3

if [ "$pdbcnt" -gt "0" ]; then
$sqlplus / as sysdba << EOF
alter session set "_oracle_script"=true;
ALTER pluggable database all open instances=all;
alter pluggable database all save state instances=all;
exit;
EOF
fi

if [ "$pdbcnt" -gt "0" ]; then
pdbname=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
set feed off;
set echo off;
set linesize 4000;
set newpage none;
select ltrim(rtrim(listagg(to_char(name),',') WITHIN GROUP (ORDER BY name))) from v\$pdbs where name not like 'PDB%SEED';
exit;
EOF1`

for x in $(echo $pdbname | tr "," "\n")
do
mkdir -p $ORACLE_BASE/oradata/$nsid/$x
$sqlplus / as sysdba << EOF
@tmpPDB.sql "$x" "$ORACLE_BASE/oradata/$nsid/$x"
EOF
done
fi

echo -e "set pagesize 100" >> $verifyDatabase
echo -e "set linesize 100" >> $verifyDatabase
echo -e "set echo off" >> $verifyDatabase
echo -e "set feedback off" >> $verifyDatabase
echo -e "set wrap on" >> $verifyDatabase
echo -e "col tablespace_name format a15" >> $verifyDatabase
echo -e "col file_name format a75" >> $verifyDatabase
echo -e "select distinct tablespace_name, file_name from dba_data_files order by 1;" >> $verifyDatabase
echo -e "select owner,count(*) from dba_tables group by owner order by 1;" >> $verifyDatabase
echo -e "select dbid, name, open_mode from v\$database;" >> $verifyDatabase
echo -e "select sum(bytes)/(1024*1024*1024) \"dbSize (GB)\" from dba_data_files;" >> $verifyDatabase
echo -e "select * from v\$instance;" >> $verifyDatabase
echo -e "exit" >> $verifyDatabase

setdb="setdb_"$nsid
echo -e "# Modify the following variables based on the environment" > $setdb
echo -e "export ORACLE_SID=$nsid" >> $setdb
echo -e "export ORACLE_HOME=$ORACLE_HOME" >> $setdb
echo -e " " >> $setdb
echo -e "export PATH=$ORACLE_HOME/bin:$PATH" >> $setdb
echo -e "#END OF FILE" >> $setdb

cp $setdb $dataDir

echo "*********************** 888888888 *********************"
export ORACLE_SID=$nsid
if [ $retval -gt 0 ]; then
echo "********** error cloning database retval:$retval ***************"
#exit 1; 
#should not reach here
fi
fi

if [ -z $hostip ]; then
   hostip="localhost"
fi

# update tnsnames.ora
#./updateTNSNames.sh $nsid $ORACLE_HOME $hostip

export ORACLE_SID=$nsid
vnum=0
vnum=`$sqlplus -S / as sysdba << 'EOF1'
set head off;
set feedback off
select count(*) from v\$instance where version like '10%';
exit;
EOF1`

set +x
syspwd="change_on_install"
if [ ! -z $ouser ] && [ "$ouser" == "sys" ] && [ ! -z $opass ]; then
   syspwd=$opass
fi

echo "*********************** 1 ********************"
echo "*********************** $nsid ********************"
export ORACLE_SID=$nsid
# create pwfile
#orapwd file=$ORACLE_HOME/dbs/orapw$nsid password=change_on_install ignorecase=yes

if [ "$vnum" -gt "0" ]; then
   orapwd file=$ORACLE_HOME/dbs/orapw$nsid password=$syspwd
else
   orapwd file=$ORACLE_HOME/dbs/orapw$nsid password=$syspwd ignorecase=yes
fi

# update /etc/oratab for reprotect
oratabentry=$nsid":"$oracleHome":Y"
echo "$oratabentry" >> /etc/oratab
fi

if [ "$recoverdb" = "true" ]; then
# grant user sysdba privilege
if [ ! -z $ouser ] && [ "$ouser" != "sys" ]; then
echo "*********************** 2 ********************"
$sqlplus / as sysdba << EOF
grant sysdba to $ouser;
exit;
EOF
fi

cp $ORACLE_HOME/dbs/orapw$nsid $dataDir

$sqlplus / as sysdba << EOF
shutdown immediate;
startup mount;
alter database archivelog;
alter database open;
exit;
EOF
fi

ORACLE_SID=$nsid
export ORACLE_SID
$sqlplus / as sysdba @verifyDatabase.sql
if [ -e $verifyDatabase ]; then
#   echo "removing $verifyDatabase before generating a new one"
   rm -f $verifyDatabase
fi
exit 0

