# This script should be run as part of Actifio Source Attribute collection

# ./collect.sh /act/scripts/output/

# Stop boto config from being parsed
export BOTO_CONFIG="/dev/null"

CONFIG_FILE="config.yaml"
SOURCE_JSON_FILE="source.json"

OUTPUT_DIR=$1
echo "OUTPUT_DIR: $OUTPUT_DIR"
if [ ! -d "$OUTPUT_DIR" ]; then
  echo "A valid and existing output directory should be provided."
  echo "Usage: ./collect full_path_to_output_folder"
  exit 1
fi

# Cd into the working directory
cd "$(dirname "$0")"
echo "Switching to collector directory"

# Check for configuration yaml file
if [ ! -f "$CONFIG_FILE" ]; then
  echo "Configuration file is missing. File: $CONFIG_FILE";
  exit 1
fi

# Check for source json file
if [ ! -f "$SOURCE_JSON_FILE" ]; then
  echo "Initial source file is missing. File: $SOURCE_JSON_FILE";
  exit 1;
fi

# Cleanup files before the run
echo "Cleaning up files before the run"
rm -rf source_*.json ser*.json *.py *.xml

# Start Attribute collection
echo "Starting RM CLI"
TMPDIR=/act/tmpdata ./cli --model_cls phoenix.base.source.Source --model_id 11111111-1111-1111-1111-111111111111 '--db.uri=file://./?single_file=false&nest_models=true' --action=run -c $CONFIG_FILE
CLI_RESULT=$?

# Move files to output folder
echo "Creating the output files"
mv source_11*.json $OUTPUT_DIR/source.json
mv source.xml $OUTPUT_DIR/.
mv collection.log $OUTPUT_DIR/.

# Cleanup files after the run
echo "Cleanup files after the run"
rm -rf source_*.json ser*.json *.py *.xml

# Exit with CLI exit code
echo "Exit Code: $CLI_RESULT"
exit $CLI_RESULT
