#!/bin/bash
set -x

#export ACT_JOBTYPE="mount"
#export ACT_PHASE="post"
SWD=/act/scripts

if [ ! -d $SWD ]; then
   echo "could not found $SWD script folder. exiting ...."
   exit 1
fi

echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ $ACT_JOBTYPE == "unmount" ] || [ $ACT_MULTI_OPNAME == "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ $ACT_JOBTYPE == "mount" ] || [ $ACT_MULTI_OPNAME == "mount" ] ; then
   if [ $ACT_PHASE == "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ "$ACT_LOGSMART_TYPE" == "log" ] ; then
        echo "** No op for log during unmount **"
     exit
   fi
   if [ $ACT_PHASE == "post" ] ; then
        echo "************************* start post during mount ************************"

source /act/scripts/act_Conf_AddToMaster_Slave.conf

###Check /act/scripts/act_Conf_AddToMaster_Slave.conf file
if [ -z $TARGET_SLAVE_NODE ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_SLAVE_NODE not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $TARGET_SLAVE_PORT ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_SLAVE_PORT not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $TARGET_POSTGRES_HOME ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_POSTGRES_HOME not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $POSTGRES_MASTER_NODE ]; then
  echo "********************* ERROR ******************************"
  echo "POSTGRES_MASTER_NODE not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $POSTGRES_MASTER_PORT ]; then
  echo "********************* ERROR ******************************"
  echo "POSTGRES_MASTER_PORT not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $POSTGRES_MASTER_DATA_PATH ]; then
  echo "********************* ERROR ******************************"
  echo "POSTGRES_MASTER_DATA_PATH not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $POSTGRES_MASTER_HOME ]; then
  echo "********************* ERROR ******************************"
  echo "POSTGRES_MASTER_HOME not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $POSTGRES_MASTER_OS_USER ]; then
  echo "********************* ERROR ******************************"
  echo "POSTGRES_MASTER_OS_USER not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $TARGET_SLAVE_MOUNTPOINT ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_SLAVE_MOUNTPOINT not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
fi
if [ -z $TARGET_POSTGRES_DATA_PATH ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_POSTGRES_DATA_PATH not set. Please check config file: /act/scripts/act_Conf_AddToMaster_Slave.conf"
  exit 1
elif [ ! -d $TARGET_POSTGRES_DATA_PATH ]; then
  echo "********************* ERROR ******************************"
  echo "Directory TARGET_POSTGRES_DATA_PATH: $TARGET_POSTGRES_DATA_PATH not exists"
  exit 1
elif [ ! -f $TARGET_POSTGRES_DATA_PATH/postgresql.conf ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_POSTGRES_DATA_PATH: $TARGET_POSTGRES_DATA_PATH is not contains postgresql.conf"
  exit 1
fi

### Change mountpoints to accortingly
#TARGET_SLAVE_MOUNTPOINT="/`echo $TARGET_SLAVE_MOUNTPOINT | cut -d/ -f2`"

### Find OSUSER and OSGroup for slave PostgreSQL
TARGET_SLAVE_OS_USER=`ls -ld $TARGET_POSTGRES_HOME | awk '{print $3}'`
TARGET_SLAVE_OS_GROUP=`ls -ld $TARGET_POSTGRES_HOME | awk '{print $4}'`

if [ $TARGET_MASTER_OS_USER == "root" ]; then
        echo "Postgresql binaris must not owned by root user."
        retval=1
        exit 1
fi

## Changes in PG_DATA
# Remove postmaster.pid
rm -f $TARGET_POSTGRES_DATA_PATH/postmaster.pid
# Change postgresql.conf
cp $TARGET_POSTGRES_DATA_PATH/postgresql.conf >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.bkp
sed '/^$\|^#\|^\t/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^listen_addresses/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^port/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^hot_standby/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^data_directory/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^hba_file/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^external_pid_file/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
echo "data_directory = '$TARGET_POSTGRES_DATA_PATH'" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
echo "listen_addresses = '$TARGET_SLAVE_NODE'" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
echo "port = $TARGET_SLAVE_PORT" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
mv $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp $TARGET_POSTGRES_DATA_PATH/postgresql.conf

# Set hot_standby on
echo "hot_standby = on" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf

# Create recover.conf in TARGET_POSTGRES_DATA_PATH
rm -f $TARGET_POSTGRES_DATA_PATH/recovery.conf
echo "standby_mode = on" >> $TARGET_POSTGRES_DATA_PATH/recovery.conf
echo "primary_conninfo = 'host=$POSTGRES_MASTER_NODE port=$POSTGRES_MASTER_PORT'" >> $TARGET_POSTGRES_DATA_PATH/recovery.conf

### Remove if tablespace_map.* files exists
rm -f $TARGET_POSTGRES_DATA_PATH/tablespace_map*

### Edit pg_hba.conf file
sed -i '/^host/d' $TARGET_POSTGRES_DATA_PATH/pg_hba.conf
sed -i '/^local/d' $TARGET_POSTGRES_DATA_PATH/pg_hba.conf
echo "local all all trust" >> $TARGET_POSTGRES_DATA_PATH/pg_hba.conf
echo "host all all $TARGET_SLAVE_NODE/32 trust" >> $TARGET_POSTGRES_DATA_PATH/pg_hba.conf

# relink tablespace files
cd $TARGET_POSTGRES_DATA_PATH/pg_tblspc
tbs=`ls`
for x in $tbs
do
  oldPath=`ls -l $x | awk '{print $NF}'`
  if [ -d $TARGET_SLAVE_MOUNTPOINT$oldPath ]; then
    unlink $x
    ln -s $TARGET_SLAVE_MOUNTPOINT$oldPath $x
    chown -R $TARGET_SLAVE_OS_USER:$TARGET_SLAVE_OS_GROUP $x
  else
    echo "Error: $TARGET_SLAVE_MOUNTPOINT$oldPath is not a directory"
    echo "Tablespace re-linking failed for $TARGET_POSTGRES_DATA_PATH/pg_tblspc/$x. "
    exit 1
  fi
done
cd $SWD

# Change permission to PG_DATA directory
firstDir="/"`echo $TARGET_SLAVE_MOUNTPOINT | cut -d/ -f2`
chmod -R 0755 $firstDir
chown -R $TARGET_SLAVE_OS_USER:$TARGET_SLAVE_OS_GROUP $TARGET_SLAVE_MOUNTPOINT
chmod -R 0700 $TARGET_POSTGRES_DATA_PATH

# Set environment for PostgreSQL
PATH=$TARGET_POSTGRES_HOME/bin:$PATH
LD_LIBRARY_PATH=$TARGET_POSTGRES_HOME/lib:$LD_LIBRARY_PATH

# start PostgreSQL
su -m $TARGET_SLAVE_OS_USER -c 'pg_ctl  -D '$TARGET_POSTGRES_DATA_PATH' start -l '$TARGET_POSTGRES_DATA_PATH'/pglog.log'
retval=$?
sleep 5

PGSTATUS=$(su -m $TARGET_SLAVE_OS_USER -c 'pg_ctl status -D '$TARGET_POSTGRES_DATA_PATH'')
if [ "$PGSTATUS" == "pg_ctl: no server running" ]; then
  echo "Error: PostgreSQL not able to start properly after restore"
  echo "Please check logfile: $TARGET_POSTGRES_DATA_PATH/pglog.log"
  tail -20 $TARGET_POSTGRES_DATA_PATH/pglog.log
  retval=1
fi

if [ $retval -eq 0 ]; then
  echo "Slave created successfully."
  echo "To complete setup please add blow line to pg_hba.conf in master and reload that server"
  echo "host	replication	$TARGET_SLAVE_OS_USER	$TARGET_SLAVE_NODE/32	trust"
else
  echo "Failed to start Postgres_Slave after restore."
  exit $retval
fi

echo "************************* end post during mount ************************"
fi
fi

exit $retval
