#!/bin/bash
set -x

#export ACT_JOBTYPE="mount"
#export ACT_PHASE="post"
SWD=/act/scripts

if [ ! -d $SWD ]; then
   echo "could not found $SWD script folder. exiting ...."
   exit 1
fi

echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ $ACT_JOBTYPE == "unmount" ] || [ $ACT_MULTI_OPNAME == "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ $ACT_JOBTYPE == "mount" ] || [ $ACT_MULTI_OPNAME == "mount" ] ; then
   if [ $ACT_PHASE == "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ "$ACT_LOGSMART_TYPE" == "log" ] ; then
        echo "** No op for log during unmount **"
     exit
   fi
   if [ $ACT_PHASE == "post" ] ; then
        echo "************************* start post during mount ************************"

source /act/scripts/act_Conf_Target_Master.conf

###Check /act/scripts/act_Conf_Target_Master.conf file
if [ -z $TARGET_MASTER_NODE ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_MASTER_NODE not set. Please check config file: /act/scripts/act_Conf_Target_Master.conf"
  exit 1
fi
if [ -z $TARGET_MASTER_PORT ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_MASTER_PORT not set. Please check config file: /act/scripts/act_Conf_Target_Master.conf"
  exit 1
fi
if [ -z $TARGET_POSTGRES_HOME ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_POSTGRES_HOME not set. Please check config file: /act/scripts/act_Conf_Target_Master.conf"
  exit 1
fi
if [ -z $TARGET_MASTER_MOUNTPOINT ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_MASTER_MOUNTPOINT not set. Please check config file: /act/scripts/act_Conf_Target_Master.conf"
  exit 1
fi
if [ -z $TARGET_POSTGRES_DATA_PATH ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_POSTGRES_DATA_PATH not set. Please check config file: /act/scripts/act_Conf_Target_Master.conf"
  exit 1
elif [ ! -d $TARGET_POSTGRES_DATA_PATH ]; then
  echo "********************* ERROR ******************************"
  echo "Directory TARGET_POSTGRES_DATA_PATH: $TARGET_POSTGRES_DATA_PATH not exists"
  exit 1
elif [ ! -f $TARGET_POSTGRES_DATA_PATH/postgresql.conf ]; then
  echo "********************* ERROR ******************************"
  echo "TARGET_POSTGRES_DATA_PATH: $TARGET_POSTGRES_DATA_PATH does not contains postgresql.conf"
  exit 1
fi

###Find PostgreSQL owner and group
TARGET_MASTER_OS_USER=`ls -ld $TARGET_POSTGRES_HOME | awk '{print $3}'`
TARGET_MASTER_OS_GROUP=`ls -ld $TARGET_POSTGRES_HOME | awk '{print $4}'`

if [ $TARGET_MASTER_OS_USER == "root" ]; then
        echo "Postgresql binaris must not owned by root user."
        retval=1
        exit 1
fi

retval=0

# Changes in PGDATA
# Remove postmaster.pid if exists.
rm -f $TARGET_POSTGRES_DATA_PATH/postmaster.pid
###Edit postgresql.conf file
cp $TARGET_POSTGRES_DATA_PATH/postgresql.conf $TARGET_POSTGRES_DATA_PATH/postgresql.conf.bkp
sed '/^$\|^#\|^\t/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^listen_addresses/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^port/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^hot_standby/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^archive_mode/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^archive_command/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^data_directory/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^hba_file/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
sed -i '/^external_pid_file/d' $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
echo "data_directory = '$TARGET_POSTGRES_DATA_PATH'" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
echo "listen_addresses = '$TARGET_MASTER_NODE'" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
echo "port = $TARGET_MASTER_PORT" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
if [[ ! -z $TARGET_MASTER_ARC_LOCATION && -d $TARGET_MASTER_ARC_LOCATION ]]; then
  echo "archive_mode = on" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
  echo "archive_command = 'test ! -f $TARGET_MASTER_ARC_LOCATION/%f && cp %p $TARGET_MASTER_ARC_LOCATION/%f'" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
else
  echo "archive_mode = off" >> $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp
fi
mv $TARGET_POSTGRES_DATA_PATH/postgresql.conf.tmp $TARGET_POSTGRES_DATA_PATH/postgresql.conf

# Remove if any recovery.conf is there.
rm -f $TARGET_POSTGRES_DATA_PATH/recovery.conf

### Remove if tablespace_map.* files exists
rm -f $TARGET_POSTGRES_DATA_PATH/tablespace_map*

### Edit pg_hba.conf file
sed -i '/^host/d' $TARGET_POSTGRES_DATA_PATH/pg_hba.conf
sed -i '/^local/d' $TARGET_POSTGRES_DATA_PATH/pg_hba.conf
echo "local all all trust" >> $TARGET_POSTGRES_DATA_PATH/pg_hba.conf
echo "host all all $TARGET_MASTER_NODE/32 trust" >> $TARGET_POSTGRES_DATA_PATH/pg_hba.conf

# relink tablespace files
cd $TARGET_POSTGRES_DATA_PATH/pg_tblspc
tbs=`ls`
for x in $tbs
do
  oldPath=`ls -l $x | awk '{print $NF}'`
  echo $oldPath
  if [ -d $TARGET_MASTER_MOUNTPOINT$oldPath ]; then
    unlink $x
    ln -s $TARGET_MASTER_MOUNTPOINT$oldPath $x
    chown -R $TARGET_MASTER_OS_USER:$TARGET_MASTER_OS_GROUP $x
  else
    echo "Warning: $TARGET_MASTER_MOUNTPOINT$oldPath is not a directory"
    echo "Tablespace re-linking failed for $TARGET_POSTGRES_DATA_PATH/pg_tblspc/$x. "
    #exit 1
  fi
done
cd $SWD

# if PITR is requested
if [ ! -z "$RECOVERY_TIME" ]; then
  sh /act/scripts/act_Post_Target_Master_PITR.sh
fi

# Change permission to PG_DATA directory
firstDir="/"`echo $TARGET_POSTGRES_DATA_PATH | cut -d"/" -f2`
chmod -R 755 $firstDir
chown -R $TARGET_MASTER_OS_USER:$TARGET_MASTER_OS_GROUP $TARGET_MASTER_MOUNTPOINT
chmod -R 0700 $TARGET_POSTGRES_DATA_PATH

# Set environment for PostgreSQL
PATH=$TARGET_POSTGRES_HOME/bin:$PATH
LD_LIBRARY_PATH=$TARGET_POSTGRES_HOME/lib

# start PostgreSQL
su -m $TARGET_MASTER_OS_USER -c 'pg_ctl -D '$TARGET_POSTGRES_DATA_PATH' start -w -l '$TARGET_POSTGRES_DATA_PATH'/pglog.log'
retval=$?

PGSTATUS=$(su -m $TARGET_MASTER_OS_USER -c 'pg_ctl status -D '$TARGET_POSTGRES_DATA_PATH'')
if [ "$PGSTATUS" == "pg_ctl: no server running" ]; then
  echo "Error: PostgreSQL not able to start properly after restore"
  echo "Please check logfile: $TARGET_POSTGRES_DATA_PATH/pglog.log"
  tail -20 $TARGET_POSTGRES_DATA_PATH/pglog.log
  exit 1 	# retval=1
fi

############################################ SuperUser and DB_Name setting ################################################
if [ -f $TARGET_POSTGRES_DATA_PATH/act_backup_details.txt ]; then
  PG_SUPER_USER=`grep SUPER_USER $TARGET_POSTGRES_DATA_PATH/act_backup_details.txt | cut -d"=" -f2`

  PROTECTED_DB_LIST=`grep PROTECTED_DB_LIST $TARGET_POSTGRES_DATA_PATH/act_backup_details.txt | cut -d"=" -f2`
  dbUpper=`awk '{ print toupper($0)}' <<< $PROTECTED_DB_LIST`

  if [[ ( ! -z $PROTECTED_DB_LIST ) && ( "$dbUpper" != "ALL" ) ]]; then
      # List of protected db list
      pDbList=`echo $PROTECTED_DB_LIST | sed "s/,/ /g"`" postgres"  	#Including dbname postgres as it is default database.

      # If there is any database renaming
      if [ ! -z $TARGET_DB_RENAME ]; then

        # Get the list of databases that are going to rename
        for x in $(echo $TARGET_DB_RENAME | sed "s/,/ /g")
        do
           RENAME_DB_LIST=$RENAME_DB_LIST" "`echo $x | cut -d":" -f1`
        done

        # Find a protected dbname that is not going to rename
        for x in $pDbList
        do
          needRename=0	# DB x not need rename
          for y in $RENAME_DB_LIST
          do
            if [[ "$x" == "$y" ]]; then 
		needRename=1; 
	        break	# DB x need rename
	    fi
          done
          if [ "$needRename" -eq "0" ]; then
            # Database $x not going to rename. Use dbname $x to connect.
            ConnDB=$x
            break
          fi
        done

        # If all protected databases are going to rename
        if [ -z ConnDB ]; then
          ConnDB=postgres
        fi
    
      # If there is no database renaming
      else
        ConnDB=`echo $pDbList | awk '{print $1}'`
      fi  

  else
    # PROTECTED_DB_LIST is ALL or null
    ConnDB=postgres
  fi

else
  ConnDB=postgres
  PG_SUPER_USER=postgres       
fi
############################################ SuperUser and DB_Name setting : Completed ################################################

#Wait until recovery
while [ "1" -eq "1" ]
do
  errMsg=$(su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "SELECT '\''1'\''"' 2>&1)
  if [[ "$errMsg" =~ "the database system is starting up" ]]; then
    sleep 5	# recovery is running
  elif [[ "$errMsg" -eq "1" ]]; then
    break;      # recovery completed
  else
    echo "ERROR: Not able to connect postgreSQL database"
    exit 1
  fi
done

sleep 10

#Create superuser with os username
su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "create user '$TARGET_MASTER_OS_USER'"'
su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "alter user '$TARGET_MASTER_OS_USER' with SUPERUSER"'
PG_SUPER_USER=$TARGET_MASTER_OS_USER

#Drop non protected databases (exclude postgres)
if [ ! -z $PROTECTED_DB_LIST ]; then
  dbUpper=`awk '{ print toupper($0)}' <<< $PROTECTED_DB_LIST`
  if [[ "$dbUpper" != "ALL" ]]; then
    pDbList=`echo $PROTECTED_DB_LIST | sed "s/,/ /g"`" postgres"
    DBList=`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "SELECT datname FROM pg_database where datistemplate = false"'`
    for x in $DBList
    do
      find=0
      for y in $pDbList
      do
        if [[ "$x" == "$y" ]]; then
 	  find=1;
	  break;
	fi
      done
      if [ "$find" -eq "0" ]; then
        echo "Drop database $x"
	su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "drop database '$x'"'
      fi
    done
  fi
fi

#Drop invalid databases
DBList=`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "SELECT datname FROM pg_database where datistemplate = false"'`
for x in $DBList
do
  tbs=`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "select t.spcname from pg_catalog.pg_database d JOIN pg_catalog.pg_tablespace t on d.dattablespace = t.oid where d.datname='\'''$x''\''"'`
  tbs=$tbs" "`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$x' -c "Select tablespace from pg_tables where tablespace is not null and tablespace not in ('\'''pg_global''\'')"'`
  for y in $tbs
  do
    tbsLoc=`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "select pg_tablespace_location(oid) from pg_tablespace where spcname in ('\'''$y''\'')"'`
    if [ ! -d $tbsLoc ]; then
      su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "drop database '$x'"'
      break
    fi
  done
done

#Drop invalid tablespaces
TBSList=`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "select spcname from pg_tablespace where spcname not in ('\''pg_default'\'','\''pg_global'\'')"'`
for x in $TBSList
do
  TBSLoc=`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "select pg_tablespace_location(oid) from pg_tablespace where spcname in ('\'''$x''\'')"'`
  if [ ! -d $TBSLoc ]; then
   su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "drop tablespace '$x'"'
  fi
done

#Rename databases if needed
if [ ! -z $TARGET_DB_RENAME ]; then
  DBs=`echo $TARGET_DB_RENAME | sed "s/,/ /g"`
  for x in $DBs
  do
    oldName=`echo $x | sed "s/:/ /g"| awk '{print $1}'`  # Case-sensitive
    newName=`echo $x | sed "s/:/ /g"| awk '{print $2}'`  # Case-sensitive
    isXdb=`su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "SELECT datname FROM pg_database where datname='\'''$oldName''\''"'`
    if [[ "$isXdb" == "$oldName" ]]; then
      su -m $TARGET_MASTER_OS_USER -c 'psql -AXt -p'$TARGET_MASTER_PORT' -U'$PG_SUPER_USER' -d'$ConnDB' -c "ALTER DATABASE \"'$oldName'\" RENAME TO \"'$newName'\""'
    else
      echo "ERROR: [$oldName] database not present."
      echo "Cannot rename database: $oldName"
    fi
  done
fi

echo "************************* end post during mount ************************"
fi
fi
exit $retval
