#!/bin/sh
# =============================================================================
# Actifio Unified Data Protector
# Copyright (c) 2011 Actifio Inc. All Rights Reserved
#
# This program code contains trade secrets of Actifio Inc. that
# (i) derive independent economic value, actual or potential, from not being
# generally known to, and not being readily ascertainable by proper means by
# other persons who can obtain economic value from their disclosure or use and;
# (ii) are the subject of efforts by Actifio Inc that are reasonable under the
# circumstances to maintain their secrecy. You may not use this program code
# without prior written authorization from Actifio Inc.
#
# Description: shell interface with RMAN client
#              takes three parameters:
#              1. login string 
#              2. rman run command
#              3. rman log output destination
#              separated by space
# rman TARGET /
# rman TARGET SYS/pwd@prod NOCATALOG  (this is the login method to be used, controlfile is used instead of catalog)
# rman TARGET / CATALOG rco/pwd@catdb 
# ==============================================================================
OLDIFS=${IFS}
IFS=','

read input
set -- $input

if [ $# -lt 6 ]; then 
   echo "invalid number of input parameters"
   exit 1
fi

login=$1
rcmd=$2
rlog=$3
sid=$4
instid=$5
authtype=$6
nlistener=$7

#orauser=`ps -ef | grep -v grep | grep ora_pmon_"$instid" | sed -e 's/^[ \t]*//' | cut -d" " -f1`
orauser=`ps -ef | grep -v grep | grep ora_pmon_"$instid$" | sed -e 's/^[ \t]*//' | cut -d" " -f1`
. /act/act_scripts/commresource.sh

#check orauser is valid
if [ ! -z "$orauser" ]; then
result=`id $orauser`
if [ $? -ne 0 ]; then
   uid=$orauser
   orauser=`$awkc -v val=$uid -F ":" '$3==val{print $1}' /etc/passwd`
   result=`id $orauser`
   if [ $? -ne 0 ]; then
      orauser=''
   fi
fi
fi

export PATH
export ORACLE_SID
export ORACLE_HOME
export TNS_ADMIN
export LD_LIBRARY_PATH

rman=$oraclehome/bin/rman

#check rman cmd is a file
echo $rcmd  | grep ".rcv"
if [ $? = 0 ]; then
    rcmd=`cat $rcmd`
fi

rmanfile="/act/act_scripts/"$sid"_rmancmd.rcv"

if [ "$os" != "Linux" ]; then
   rcmd=`echo "$rcmd" | sed 's/;/;\\\n/g'`
fi

set +x
if [ ! "$authtype" = "os" ]; then
psph=`/opt/act/bin/udsagent decode -d $sid dp`
login=`echo $login | sed "s/xxx/${psph}/g"`
fi

echo "$login" | grep -iw CATALOG > /dev/null 2>&1
if [ $? -eq 0 ]; then
csph=`/opt/act/bin/udsagent decode -d $sid cp`
login=`echo $login | sed "s/zzz/${csph}/g"`
fi

echo "$login" | grep -wi NOCATALOG >/dev/null 2>&1
if [ $? = 0 ]; then
   login=`echo $login | sed "s/NOCATALOG//g"`
   echo "$login" | grep -i sysbackup > /dev/null 2>&1
   if [ $? = 0 ]; then
     echo "connect target '$login'" > $rmanfile
   else
      echo "connect target  $login" > $rmanfile
   fi

else
   echo "$login" | grep -i sysbackup > /dev/null 2>&1
   if [ $? = 0 ]; then
      tgt=`echo "$login" | tr -s " " | cut -d" " -f1-3`
      echo "connect target '$tgt'" > $rmanfile
      catal=`echo "$login" | tr -s " " | cut -d" " -f4-5`
      echo "connect $catal" >> $rmanfile
   else
      tgt=`echo "$login" | tr -s " " | cut -d" " -f1`
      echo "connect target $tgt" > $rmanfile
      catal=`echo "$login" | tr -s " " | cut -d" " -f2-3`
      echo "connect $catal" >> $rmanfile
   fi
fi

echo "$rcmd" >> $rmanfile

if [ "$os" = "Linux" ]; then
sed -i 's/;/;\
/g' $rmanfile
fi

chmod 644 $rmanfile

#check log file
if [ ! -f "$rlog" ]; then
   touch $rlog
fi

if [ "$os" = "SunOS" ]; then
    #get group for the user
    grp=`/usr/xpg4/bin/id -gn $orauser`
    if [ ! -z "$grp" ]; then
       chown $orauser:$grp $rlog
       chmod 755 $rlog
    fi 
else
   . /act/act_scripts/oracleclone/OracleAppMountComFunc.sh
   set +x
   change_own $orauser $rlog
   set +x
fi

NLS_DATE_FORMAT="yyyymmddhh24mi"
NLS_LANG=AMERICAN_AMERICA.US7ASCII
export NLS_DATE_FORMAT
export NLS_LANG
echo $rcmd | egrep -i 'show controlfile autobackup|show retention policy' >/dev/null 2>&1
if [ $? = 0 ]; then
   if [ ! "$authtype" = "os" ]; then
       retval=`$rman @$rmanfile`
    else
       if [ "$os" != "Linux" ]; then
          retval=`su $orauser -c "$rman @$rmanfile"`
       else
          retval=`su -m $orauser -c "$rman @$rmanfile"`
       fi
    fi
else
   if [ ! "$authtype" = "os" ]; then
      retval=`$rman @$rmanfile log=$rlog append`
   else
      #os auth, need to check orauser
      if [ -z "$orauser" ]; then
        echo "Could not determine os user for $sid"
        exit 1
      fi
      #check orauser

      if [ "$os" != "Linux" ]; then
        retval=`su $orauser -c "$rman @$rmanfile log=$rlog append"`
      else
        retval=`su -m $orauser -c "$rman @$rmanfile log=$rlog append"`
      fi 
   fi
fi

status=$?

if [ $status -ne 0 ]; then
    echo "Error: $revtal"
else
    echo $retval
fi

IFS=$OLDIFS

rm $rmanfile
exit $status
 
