#!/bin/bash
set -x

DBSID=$1
DBUSER=$2
DBPORT=$3
PORT=$4
DELLOG=$5
HANAVERSION=$6
TDBUSER=$7

DATA=`date`
echo "************************** DATE:$DATE: start log delete post script for $DBSID database ******************"

if [ -z "$HANAVERSION" ]; then
   echo "************* HANAVERSION is missing *************"
   echo "check appid.xxx for setting the right HANAVERSION"
   exit 1
fi


if [ "$HANAVERSION" = "1.0" ]
then
SQL="SELECT TOP 1 min(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME >= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME = 'data snapshot' and STATE_NAME = 'successful' "
ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i 00 -U $DBUSER -a -j $SQL | head -n1`
ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

if [ "$ID" = "?" ]
then
 echo "********** !!! WARNING !!!: no snapshot ID found to delete the log for database older then $DELLOG *********"
 exit
fi

SQL="BACKUP CATALOG DELETE ALL BEFORE BACKUP_ID $ID COMPLETE "
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $SQL

else

if [ -z "$TDBUSER" ]; then
   echo "***************** TENANT database key name is missing ****************"
   echo "check appid.xxx for setting the right TENANT database key name"
   exit 1
fi

SQL="SELECT TOP 1 min(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME >= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME = 'data snapshot' and STATE_NAME = 'successful' "
ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i 00 -U $DBUSER -a -j $SQL | head -n1`
ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

if [ "$ID" = "?" ]
then
 echo "********** !!! WARNING !!!: no snapshot ID found to delete the log for systemdb database for older then $DELLOG *********"
 exit
fi

echo "************************** delete systemdb log ******************"
SQL="BACKUP CATALOG DELETE FOR SYSTEMDB ALL BEFORE BACKUP_ID $ID COMPLETE "
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $DBUSER -a -j $SQL

echo "************************** delete tenant log ******************"

SQL="SELECT TOP 1 min(to_bigint(BACKUP_ID)) FROM SYS.M_BACKUP_CATALOG where SYS_START_TIME >= ADD_DAYS(CURRENT_TIMESTAMP, -$DELLOG) and ENTRY_TYPE_NAME = 'data snapshot' and STATE_NAME = 'successful' "
ID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i 00 -U $TDBUSER -a -j $SQL | head -n1`
ID=`echo ${ID} | sed -e 's/^[ \t]*//'`

SQLT="SELECT TOP 1 DATABASE_NAME from M_DATABASES where DATABASE_NAME <> 'SYSTEMDB'"
TSID=`/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i 00 -U $TDBUSER -a -j $SQLT | head -n1`
TSID=`echo "$TSID" | sed -e 's/^"//' -e 's/"$//'`

if [ "$ID" = "?" ]
then
 echo "********** !!! WARNING !!!: no snapshot ID found to delete the log for $TSID database for older then $DELLOG *********"
 exit
fi

SQL="BACKUP CATALOG DELETE ALL BEFORE BACKUP_ID $ID COMPLETE "
/usr/sap/$DBSID/$DBPORT/exe/hdbsql -i $PORT -U $TDBUSER -a -j $SQL
fi
echo "************************** end log delete post script ******************"
exit

