#!/bin/sh

export ACT_JOBTYPE="mount"
export ACT_PHASE="post"
tmp_loc=/act/touch

if [ ! -d "$tmp_loc" ]; then
   mkdir -p $tmp_loc
fi
chmod 777 /act/touch

set -x
cd /act/scripts
conffile=/act/scripts/act_Conf_Target_Primary.conf
source $conffile
echo "***************** POSTSKJOB_TYPE= $ACT_JOBTYPE *********************"

if [ "$ACT_JOBTYPE" = "unmount" ] || [ "$ACT_MULTI_OPNAME" = "unmount" ] ; then
   echo "NO-OP for post-script when unmount"
   exit 0
fi
if [ $ACT_JOBTYPE = "mount" ] || [ $ACT_MULTI_OPNAME = "mount" ] ; then
   if [ $ACT_PHASE = "pre" ] ; then
        echo "** No op for pre during unmount **"
      exit
   fi
   if [ $ACT_PHASE = "post" ] ; then
        echo "************************* start post during mount ************************"

mntpt=$TARGET_MNT_PNT
chown -R $OSUSER:$OSUSER /$mntpt

if  [ ! -z "$NEWDBNAME_PREFIX" ] && [ ! -z "$TARGET_DBNAME_LIST" ]; then
echo "Specify either NEWEDBNAME_PREFIX or TARGET_DBNAME_LIST variables..!!"
exit 1
fi

#flag=`echo $SRC_DBNAME | grep "," | wc -l`

#if [ "$flag" -eq "1" ]; then
# SRC_DBNAME=`echo $SRC_DBNAME | awk -F"," '{print $1}'`
#fi

#Get the Database and Device information from Manifest file created during the backup
echo "Listing databases and devices.."

su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -e -n -b -o $tmp_loc/devices_$SRC_DBNAME.txt" <<EOF
set nocount on
go
mount database all from '$TARGET_MNT_PNT$MANIFEST_FILE_LOC/Manifest_File' with listonly
go
quit
EOF

retval=$?
if [ $retval -gt 0 ]; then
 echo "ERROR: MOUNT LISTONLY Command failed!"
 exit 1
fi

db_names=`awk '/database/{flag=1; next} /device/{flag=0} flag' $tmp_loc/devices_$SRC_DBNAME.txt`

for i in $db_names
do
 if [ -z "$db_list" ]; then
   db_list=$i
 else
  db_list=$db_list,$i
 fi
done 

sed '1,/device/d' $tmp_loc/devices_$SRC_DBNAME.txt > $tmp_loc/mount_device_$SRC_DBNAME.txt
sed -i "s/'/\\${mntpt}/" $tmp_loc/mount_device_$SRC_DBNAME.txt
sed -i "s/^.*$/'&/" $tmp_loc/mount_device_$SRC_DBNAME.txt


while read line
do
if [ -z "$device_listcomma" ]; then
   device_listcomma=$line
else
   device_listcomma=$device_listcomma","$line
fi
done < $tmp_loc/mount_device_$SRC_DBNAME.txt
echo $device_listcomma

#Check if the devie already exists

while read line
do
 check_device_exists=`echo $line | awk -F"=" '{print $2}'`
 su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X -n -b -o $tmp_loc/device_exists.txt" <<EOF
set nocount on
go
select name from master..sysdevices where name=$check_device_exists
go
quit
EOF
 check_device_exists_flag=`cat $tmp_loc/device_exists.txt | wc -l`
 if [ $check_device_exists_flag -gt "0" ]; then
   echo "ERROR: Device already exists on the target server!"
   exit 1
 fi
done <$tmp_loc/mount_device_$SRC_DBNAME.txt

# Mount Database, Recover and Online
echo " Mounting... $device_listcomma"
su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X --retserverror" <<EOF
use master
go
mount database all from '$TARGET_MNT_PNT$MANIFEST_FILE_LOC/Manifest_File' 
using $device_listcomma
go
EOF
retval=$?

if [ $retval -gt 0 ]; then
 echo "ERROR: Mount database command failed!"
 exit 1
fi

# Performing Log Recovery

if [ "$LOG_RECOVERY" = "YES" ]; then
 /act/scripts/act_sybase_logrecovery.sh TRUE
 if [ "$?" -gt 0 ]; then
   echo "WARNING: Log Recovery Failed!. Continuing with ONLINE Database command."
 fi
fi

for i in $db_names
do
su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X --retserverror" <<EOF
use master
go
online database $i
go
EOF
done

retval=$?
if [ $retval -gt 0 ]; then
 echo "ERROR: Online database command failed!."
 exit 1
fi

# Rename Device
echo " Renaming Logical name of the device.."
tmpmntpt=`echo $mntpt | sed -e "s/\///g"`
while read line
do
device_name=`echo $line |awk -F "'" '{print $4}'`
device_newname=$tmpmntpt"_"$device_name
echo $device_newname
su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X --retserverror" <<EOF
use master
go
sp_configure 'allow updates',1 
go
update sysdevices set name = '$device_newname' where name = '$device_name'
go
EOF
done < $tmp_loc/mount_device_$SRC_DBNAME.txt


if [ $? -gt "0" ]; then
 echo "ERROR: Renaming devices. Exiting!"
fi
#Rename Database


#Rename when Prefix is specified


if [ ! -z "$NEWDBNAME_PREFIX" ]; then
echo "Rename operation started....!"
echo "SRC:$SRC_DBNAME"

for i in $(echo $SRC_DBNAME | sed "s/,/ /g")
do
    db_newname=$NEWDBNAME_PREFIX"_"$i
    db_oldname=$i
su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME  -X --retserverror" <<EOF
sp_dboption $db_oldname,single,true
go
use $db_oldname
go
checkpoint
go
use master
go
sp_renamedb $db_oldname,$db_newname
go
sp_dboption $db_newname,single,false
go
use $db_newname
go
checkpoint
go
EOF
done
fi

if [ $? -gt 0 ]; then
  echo "ERROR: Database rename operation failed.!"
  exit 1
fi


if [ ! -z "$TARGET_DBNAME_LIST" ]; then
echo "Rename operation started....!"
for i in $(echo $TARGET_DBNAME_LIST | sed "s/,/ /g")
do
    db_newname=`echo $i |awk -F":" '{print $2}'`
    db_oldname=`echo $i |awk -F":" '{print $1}'`
su - $OSUSER -c "$TARGET_SYBASE_SQLD/bin/isql -U$TARGET_DB_USER -P$TARGET_DBUSER_PASSWD -S$TARGET_SERVER_NAME -X --retserverror" <<EOF
sp_dboption $db_oldname,single,true
go
use $db_oldname
go
checkpoint
go
use master
go
sp_renamedb $db_oldname,$db_newname
go
sp_dboption $db_newname,single,false
go
use $db_newname
go
checkpoint
go
EOF
done

fi
if [ $? -gt 0 ]; then
 echo "ERROR: Renaming the databases!"
fi

if [ -e $tmp_loc/mount_device_$SRC_DBNAME.txt ]; then
 rm -f $tmp_loc/mount_device_$SRC_DBNAME.txt
fi

if [ -e $tmp_loc/devices_$SRC_DBNAME.txt ]; then
 rm -f $tmp_loc/devices_$SRC_DBNAME.txt
fi

if [ -e $tmp_loc/device_exists.txt ]; then
 rm -f $tmp_loc/device_exists.txt
fi

fi
fi

exit $retval
